# Copyright 2023 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import os
import sys
from unittest import mock

import pytest

from tests import AWSRequest, CLIRunner, HTTPResponse, SessionStubber


@pytest.fixture
def session_stubber():
    return SessionStubber()


@pytest.fixture
def region():
    return "us-west-2"


@pytest.fixture
def aws_config_file(tmp_path):
    config_file = os.path.join(tmp_path, 'aws-config-file-for-ssh-command')
    with open(config_file, 'wb') as fd:
        fd.write(
            b'[profile test-profile]\n'
            b'aws_access_key_id=access_key\n'
            b'aws_secret_access_key=secret_key\n'
            b'[profile test-fips]\n'
            b'use_fips_endpoint=true\n'
            b'aws_access_key_id=access_key\n'
            b'aws_secret_access_key=secret_key\n'
        )
        return config_file


@pytest.fixture
def cli_runner(session_stubber, region, aws_config_file):
    cli_runner = CLIRunner(session_stubber=session_stubber)
    cli_runner.env["AWS_DEFAULT_REGION"] = region
    cli_runner.env["AWS_CONFIG_FILE"] = aws_config_file
    return cli_runner


def get_describe_private_instance_response():
    return """
     <DescribeInstancesResponse xmlns="http://ec2.amazonaws.com/doc/2016-11-15/">
         <reservationSet>
             <item>
                 <instancesSet>
                     <item>
                         <instanceId>i-123</instanceId>
                         <subnetId>subnet-123</subnetId>
                         <vpcId>vpc-123</vpcId>
                         <privateIpAddress>10.0.0.0</privateIpAddress>
                     </item>
                 </instancesSet>
             </item>
         </reservationSet>
     </DescribeInstancesResponse>
     """


@pytest.fixture
def describe_private_instance_response():
    return get_describe_private_instance_response()


def get_describe_public_instance_response():
    return """
     <DescribeInstancesResponse xmlns="http://ec2.amazonaws.com/doc/2016-11-15/">
         <reservationSet>
             <item>
                 <instancesSet>
                     <item>
                         <instanceId>i-123</instanceId>
                         <subnetId>subnet-123</subnetId>
                         <vpcId>vpc-123</vpcId>
                         <ipAddress>11.11.11.11</ipAddress>
                         <privateIpAddress>10.0.0.0</privateIpAddress>
                         <ipv6Addresses>2600:1f10:4f8e:db01:73f5:6b9d:c0da:1c27</ipv6Addresses>
                     </item>
                 </instancesSet>
             </item>
         </reservationSet>
     </DescribeInstancesResponse>
     """


@pytest.fixture
def describe_public_instance_response():
    return get_describe_public_instance_response()


def get_describe_ipv6_instance_response():
    return """
     <DescribeInstancesResponse xmlns="http://ec2.amazonaws.com/doc/2016-11-15/">
         <reservationSet>
             <item>
                 <instancesSet>
                     <item>
                         <instanceId>i-123</instanceId>
                         <subnetId>subnet-123</subnetId>
                         <vpcId>vpc-123</vpcId>
                         <privateIpAddress>10.0.0.0</privateIpAddress>
                         <ipv6Address>2600:1f10:4f8e:db01:73f5:6b9d:c0da:1c27</ipv6Address>
                     </item>
                 </instancesSet>
             </item>
         </reservationSet>
     </DescribeInstancesResponse>
     """


@pytest.fixture
def describe_ipv6_instance_response():
    return get_describe_ipv6_instance_response()


@pytest.fixture
def describe_no_ip_instance_response():
    return """
     <DescribeInstancesResponse xmlns="http://ec2.amazonaws.com/doc/2016-11-15/">
         <reservationSet>
             <item>
                 <instancesSet>
                     <item>
                         <instanceId>i-123</instanceId>
                         <subnetId>subnet-123</subnetId>
                         <vpcId>vpc-123</vpcId>
                     </item>
                 </instancesSet>
             </item>
         </reservationSet>
     </DescribeInstancesResponse>
     """


@pytest.fixture
def send_ssh_public_key_response():
    return (
        b'{"RequestId":"01adeb55-3f80-4736-b789-d57feb69691d","Success":true}'
    )


@pytest.fixture
def request_params_for_describe_instance():
    return {'InstanceIds': ['i-123']}


def get_describe_eice_response():
    return """
    <DescribeInstanceConnectEndpointsResponse xmlns="http://ec2.amazonaws.com/doc/2016-11-15/">
        <instanceConnectEndpointSet>
            <item>
                <dnsName>dns.com</dnsName>
                <fipsDnsName>fips.dns.com</fipsDnsName>
                <instanceConnectEndpointId>eice-123</instanceConnectEndpointId>
                <state>create-complete</state>
                <subnetId>subnet-123</subnetId>
                <vpcId>vpc-123</vpcId>
            </item>
        </instanceConnectEndpointSet>
    </DescribeInstanceConnectEndpointsResponse>
    """


def get_describe_eice_response_with_state(state):
    return f"""
    <DescribeInstanceConnectEndpointsResponse xmlns="http://ec2.amazonaws.com/doc/2016-11-15/">
        <instanceConnectEndpointSet>
            <item>
                <dnsName>dns.com</dnsName>
                <fipsDnsName>fips.dns.com</fipsDnsName>
                <instanceConnectEndpointId>eice-123</instanceConnectEndpointId>
                <state>{state}</state>
                <subnetId>subnet-123</subnetId>
                <vpcId>vpc-123</vpcId>
            </item>
        </instanceConnectEndpointSet>
    </DescribeInstanceConnectEndpointsResponse>
    """


def get_describe_eice_response_create_complete():
    return get_describe_eice_response_with_state("create-complete")


def get_describe_eice_response_update_in_progress():
    return get_describe_eice_response_with_state("update-in-progress")


def get_describe_eice_response_update_complete():
    return get_describe_eice_response_with_state("update-complete")


def get_describe_eice_response_update_failed():
    return get_describe_eice_response_with_state("update-failed")


@pytest.fixture
def describe_eice_response():
    return get_describe_eice_response()


def get_request_params_for_describe_eice():
    return {
        'Filters': [
            {
                'Name': 'state',
                'Values': [
                    'create-complete',
                    'update-in-progress',
                    'update-failed',
                    'update-complete',
                ],
            },
            {'Name': 'vpc-id', 'Values': ['vpc-123']},
        ]
    }


def get_request_params_for_describe_eice_with_eice_id():
    return {
        'Filters': [
            {
                'Name': 'state',
                'Values': [
                    'create-complete',
                    'update-in-progress',
                    'update-failed',
                    'update-complete',
                ],
            }
        ],
        'InstanceConnectEndpointIds': ['eice-12345'],
    }


class TestSSHCommand:
    @pytest.mark.parametrize(
        "describe_instance_response,describe_eice_response,request_params_for_describe_eice,cli_input,expected_ssh_command",
        [
            pytest.param(
                get_describe_private_instance_response(),
                get_describe_eice_response(),
                get_request_params_for_describe_eice(),
                [
                    "ec2-instance-connect",
                    "ssh",
                    "--instance-id",
                    "i-123",
                    "--private-key-file",
                    "/tmp/ssh-file",
                ],
                [
                    'ssh',
                    '-o',
                    'ServerAliveInterval=5',
                    '-p',
                    '22',
                    '-i',
                    '/tmp/ssh-file',
                    '-o',
                    'ProxyCommand=aws ec2-instance-connect open-tunnel --instance-id i-123 '
                    '--private-ip-address 10.0.0.0 --remote-port 22 '
                    '--instance-connect-endpoint-id eice-123 --instance-connect-endpoint-dns-name dns.com',
                    'ec2-user@10.0.0.0',
                ],
                id='Open-Tunnel: connect by instance id + key',
            ),
            pytest.param(
                get_describe_private_instance_response(),
                get_describe_eice_response(),
                get_request_params_for_describe_eice_with_eice_id(),
                [
                    "ec2-instance-connect",
                    "ssh",
                    "--private-key-file",
                    "/tmp/ssh-file",
                    "--instance-id",
                    "i-123",
                    "--eice-options",
                    "endpointId=eice-12345",
                ],
                [
                    'ssh',
                    '-o',
                    'ServerAliveInterval=5',
                    '-p',
                    '22',
                    '-i',
                    '/tmp/ssh-file',
                    '-o',
                    'ProxyCommand=aws ec2-instance-connect open-tunnel --instance-id i-123 '
                    '--private-ip-address 10.0.0.0 --remote-port 22 '
                    '--instance-connect-endpoint-id eice-123 --instance-connect-endpoint-dns-name dns.com',
                    'ec2-user@10.0.0.0',
                ],
                id='Open-Tunnel: connect to specific EICE',
            ),
            pytest.param(
                get_describe_private_instance_response(),
                None,
                None,
                [
                    "ec2-instance-connect",
                    "ssh",
                    "--private-key-file",
                    "/tmp/ssh-file",
                    "--instance-id",
                    "i-123",
                    "--eice-options",
                    "endpointId=eice-12345,dnsName=dns.domain.com",
                ],
                [
                    'ssh',
                    '-o',
                    'ServerAliveInterval=5',
                    '-p',
                    '22',
                    '-i',
                    '/tmp/ssh-file',
                    '-o',
                    'ProxyCommand=aws ec2-instance-connect open-tunnel --instance-id i-123 '
                    '--private-ip-address 10.0.0.0 --remote-port 22 '
                    '--instance-connect-endpoint-id eice-12345 --instance-connect-endpoint-dns-name dns.domain.com',
                    'ec2-user@10.0.0.0',
                ],
                id="Open-Tunnel: connect by eice id + domain",
            ),
            pytest.param(
                get_describe_private_instance_response(),
                get_describe_eice_response(),
                get_request_params_for_describe_eice(),
                [
                    "ec2-instance-connect",
                    "ssh",
                    "--private-key-file",
                    "/tmp/ssh-file",
                    "--instance-id",
                    "i-123",
                    "--eice-options",
                    "maxTunnelDuration=61",
                ],
                [
                    'ssh',
                    '-o',
                    'ServerAliveInterval=5',
                    '-p',
                    '22',
                    '-i',
                    '/tmp/ssh-file',
                    '-o',
                    'ProxyCommand=aws ec2-instance-connect open-tunnel --instance-id i-123 '
                    '--private-ip-address 10.0.0.0 --remote-port 22 '
                    '--instance-connect-endpoint-id eice-123 --instance-connect-endpoint-dns-name dns.com '
                    '--max-tunnel-duration 61',
                    'ec2-user@10.0.0.0',
                ],
                id="Open-Tunnel: allow customer to define maxTunnelDuration",
            ),
            pytest.param(
                get_describe_private_instance_response(),
                get_describe_eice_response(),
                get_request_params_for_describe_eice(),
                [
                    "ec2-instance-connect",
                    "ssh",
                    "--private-key-file",
                    "/tmp/ssh-file",
                    "--instance-id",
                    "i-123",
                    "--ssh-port",
                    "24",
                ],
                [
                    'ssh',
                    '-o',
                    'ServerAliveInterval=5',
                    '-p',
                    '24',
                    '-i',
                    '/tmp/ssh-file',
                    '-o',
                    'ProxyCommand=aws ec2-instance-connect open-tunnel --instance-id i-123 '
                    '--private-ip-address 10.0.0.0 --remote-port 24 '
                    '--instance-connect-endpoint-id eice-123 --instance-connect-endpoint-dns-name dns.com',
                    'ec2-user@10.0.0.0',
                ],
                id="Open-Tunnel: allow customer to define ssh port",
            ),
            pytest.param(
                get_describe_private_instance_response(),
                get_describe_eice_response(),
                get_request_params_for_describe_eice(),
                [
                    "ec2-instance-connect",
                    "ssh",
                    "--private-key-file",
                    "/tmp/ssh-file",
                    "--instance-id",
                    "i-123",
                    "--os-user",
                    "new-user",
                ],
                [
                    'ssh',
                    '-o',
                    'ServerAliveInterval=5',
                    '-p',
                    '22',
                    '-i',
                    '/tmp/ssh-file',
                    '-o',
                    'ProxyCommand=aws ec2-instance-connect open-tunnel --instance-id i-123 '
                    '--private-ip-address 10.0.0.0 --remote-port 22 '
                    '--instance-connect-endpoint-id eice-123 --instance-connect-endpoint-dns-name dns.com',
                    'new-user@10.0.0.0',
                ],
                id='Open-Tunnel: allow customer to define ssh user',
            ),
            pytest.param(
                get_describe_private_instance_response(),
                get_describe_eice_response(),
                get_request_params_for_describe_eice(),
                [
                    "ec2-instance-connect",
                    "ssh",
                    "--private-key-file",
                    "/tmp/ssh-file",
                    "--instance-id",
                    "i-123",
                    "--local-forwarding",
                    "8080:localhost:9090",
                ],
                [
                    'ssh',
                    '-o',
                    'ServerAliveInterval=5',
                    '-p',
                    '22',
                    '-i',
                    '/tmp/ssh-file',
                    '-L',
                    '8080:localhost:9090',
                    '-o',
                    'ProxyCommand=aws ec2-instance-connect open-tunnel --instance-id i-123 '
                    '--private-ip-address 10.0.0.0 --remote-port 22 '
                    '--instance-connect-endpoint-id eice-123 --instance-connect-endpoint-dns-name dns.com',
                    'ec2-user@10.0.0.0',
                ],
                id='Open-Tunnel: allow customer to local-forwarding',
            ),
            pytest.param(
                get_describe_private_instance_response(),
                get_describe_eice_response(),
                get_request_params_for_describe_eice(),
                [
                    "ec2-instance-connect",
                    "ssh",
                    "--instance-id",
                    "i-123",
                    "--private-key-file",
                    "/tmp/ssh-file",
                    "--region",
                    "us-east-2",
                    "--profile",
                    "test-profile",
                ],
                [
                    'ssh',
                    '-o',
                    'ServerAliveInterval=5',
                    '-p',
                    '22',
                    '-i',
                    '/tmp/ssh-file',
                    '-o',
                    'ProxyCommand=aws ec2-instance-connect open-tunnel --instance-id i-123 '
                    '--private-ip-address 10.0.0.0 --remote-port 22 --region us-east-2 --profile test-profile '
                    '--instance-connect-endpoint-id eice-123 --instance-connect-endpoint-dns-name dns.com',
                    'ec2-user@10.0.0.0',
                ],
                id='Open-Tunnel: pass global parameters to sub-command',
            ),
            pytest.param(
                get_describe_public_instance_response(),
                get_describe_eice_response(),
                get_request_params_for_describe_eice(),
                [
                    "ec2-instance-connect",
                    "ssh",
                    "--instance-id",
                    "i-123",
                    "--private-key-file",
                    "/tmp/ssh-file",
                    "--connection-type",
                    "eice",
                ],
                [
                    'ssh',
                    '-o',
                    'ServerAliveInterval=5',
                    '-p',
                    '22',
                    '-i',
                    '/tmp/ssh-file',
                    '-o',
                    'ProxyCommand=aws ec2-instance-connect open-tunnel --instance-id i-123 '
                    '--private-ip-address 10.0.0.0 --remote-port 22 '
                    '--instance-connect-endpoint-id eice-123 --instance-connect-endpoint-dns-name dns.com',
                    'ec2-user@10.0.0.0',
                ],
                id='Open-Tunnel: select private IP when connection type eice',
            ),
            pytest.param(
                get_describe_private_instance_response(),
                get_describe_eice_response(),
                get_request_params_for_describe_eice(),
                [
                    "ec2-instance-connect",
                    "ssh",
                    "--instance-id",
                    "i-123",
                    "--private-key-file",
                    "/tmp/ssh-file",
                    "--profile",
                    "test-fips",
                ],
                [
                    'ssh',
                    '-o',
                    'ServerAliveInterval=5',
                    '-p',
                    '22',
                    '-i',
                    '/tmp/ssh-file',
                    '-o',
                    'ProxyCommand=aws ec2-instance-connect open-tunnel --instance-id i-123 '
                    '--private-ip-address 10.0.0.0 --remote-port 22 --profile test-fips '
                    '--instance-connect-endpoint-id eice-123 '
                    '--instance-connect-endpoint-dns-name fips.dns.com',
                    'ec2-user@10.0.0.0',
                ],
                id='Open-Tunnel: connect by instance id + key to fips endpoint',
            ),
            pytest.param(
                get_describe_public_instance_response(),
                get_describe_eice_response(),
                get_request_params_for_describe_eice(),
                [
                    "ec2-instance-connect",
                    "ssh",
                    "--instance-id",
                    "i-123",
                    "--private-key-file",
                    "/tmp/ssh-file",
                    "--eice-options",
                    "maxTunnelDuration=61",
                ],
                [
                    'ssh',
                    '-o',
                    'ServerAliveInterval=5',
                    '-p',
                    '22',
                    '-i',
                    '/tmp/ssh-file',
                    '-o',
                    'ProxyCommand=aws ec2-instance-connect open-tunnel --instance-id i-123 '
                    '--private-ip-address 10.0.0.0 --remote-port 22 '
                    '--instance-connect-endpoint-id eice-123 --instance-connect-endpoint-dns-name dns.com '
                    '--max-tunnel-duration 61',
                    'ec2-user@10.0.0.0',
                ],
                id='Open-Tunnel: use private ip and eice when eice options defined without connection-type',
            ),
            pytest.param(
                get_describe_public_instance_response(),
                get_describe_eice_response(),
                get_request_params_for_describe_eice(),
                [
                    "ec2-instance-connect",
                    "ssh",
                    "--instance-id",
                    "i-123",
                    "--private-key-file",
                    "/tmp/ssh-file",
                    "--connection-type",
                    "eice",
                    "--instance-ip",
                    "1.10.10.10",
                ],
                [
                    'ssh',
                    '-o',
                    'ServerAliveInterval=5',
                    '-p',
                    '22',
                    '-i',
                    '/tmp/ssh-file',
                    '-o',
                    'ProxyCommand=aws ec2-instance-connect open-tunnel --instance-id i-123 '
                    '--private-ip-address 1.10.10.10 --remote-port 22 '
                    '--instance-connect-endpoint-id eice-123 --instance-connect-endpoint-dns-name dns.com',
                    'ec2-user@1.10.10.10',
                ],
                id='Open-Tunnel: use provided ip',
            ),
            pytest.param(
                get_describe_private_instance_response(),
                get_describe_eice_response_create_complete(),
                get_request_params_for_describe_eice(),
                [
                    "ec2-instance-connect",
                    "ssh",
                    "--instance-id",
                    "i-123",
                    "--private-key-file",
                    "/tmp/ssh-file",
                ],
                [
                    'ssh',
                    '-o',
                    'ServerAliveInterval=5',
                    '-p',
                    '22',
                    '-i',
                    '/tmp/ssh-file',
                    '-o',
                    'ProxyCommand=aws ec2-instance-connect open-tunnel --instance-id i-123 '
                    '--private-ip-address 10.0.0.0 --remote-port 22 '
                    '--instance-connect-endpoint-id eice-123 --instance-connect-endpoint-dns-name dns.com',
                    'ec2-user@10.0.0.0',
                ],
                id='Open-Tunnel: connect via eice in create-complete',
            ),
            pytest.param(
                get_describe_private_instance_response(),
                get_describe_eice_response_update_in_progress(),
                get_request_params_for_describe_eice(),
                [
                    "ec2-instance-connect",
                    "ssh",
                    "--instance-id",
                    "i-123",
                    "--private-key-file",
                    "/tmp/ssh-file",
                ],
                [
                    'ssh',
                    '-o',
                    'ServerAliveInterval=5',
                    '-p',
                    '22',
                    '-i',
                    '/tmp/ssh-file',
                    '-o',
                    'ProxyCommand=aws ec2-instance-connect open-tunnel --instance-id i-123 '
                    '--private-ip-address 10.0.0.0 --remote-port 22 '
                    '--instance-connect-endpoint-id eice-123 --instance-connect-endpoint-dns-name dns.com',
                    'ec2-user@10.0.0.0',
                ],
                id='Open-Tunnel: connect via eice in update-in-progress',
            ),
            pytest.param(
                get_describe_private_instance_response(),
                get_describe_eice_response_update_complete(),
                get_request_params_for_describe_eice(),
                [
                    "ec2-instance-connect",
                    "ssh",
                    "--instance-id",
                    "i-123",
                    "--private-key-file",
                    "/tmp/ssh-file",
                ],
                [
                    'ssh',
                    '-o',
                    'ServerAliveInterval=5',
                    '-p',
                    '22',
                    '-i',
                    '/tmp/ssh-file',
                    '-o',
                    'ProxyCommand=aws ec2-instance-connect open-tunnel --instance-id i-123 '
                    '--private-ip-address 10.0.0.0 --remote-port 22 '
                    '--instance-connect-endpoint-id eice-123 --instance-connect-endpoint-dns-name dns.com',
                    'ec2-user@10.0.0.0',
                ],
                id='Open-Tunnel: connect via eice in update-complete',
            ),
            pytest.param(
                get_describe_private_instance_response(),
                get_describe_eice_response_update_failed(),
                get_request_params_for_describe_eice(),
                [
                    "ec2-instance-connect",
                    "ssh",
                    "--instance-id",
                    "i-123",
                    "--private-key-file",
                    "/tmp/ssh-file",
                ],
                [
                    'ssh',
                    '-o',
                    'ServerAliveInterval=5',
                    '-p',
                    '22',
                    '-i',
                    '/tmp/ssh-file',
                    '-o',
                    'ProxyCommand=aws ec2-instance-connect open-tunnel --instance-id i-123 '
                    '--private-ip-address 10.0.0.0 --remote-port 22 '
                    '--instance-connect-endpoint-id eice-123 --instance-connect-endpoint-dns-name dns.com',
                    'ec2-user@10.0.0.0',
                ],
                id='Open-Tunnel: connect via eice in update-failed',
            ),
            pytest.param(
                get_describe_public_instance_response(),
                None,
                None,
                [
                    "ec2-instance-connect",
                    "ssh",
                    "--instance-id",
                    "i-123",
                    "--private-key-file",
                    "/tmp/ssh-file",
                    "--ssh-port",
                    "24",
                ],
                [
                    'ssh',
                    '-o',
                    'ServerAliveInterval=5',
                    '-p',
                    '24',
                    '-i',
                    '/tmp/ssh-file',
                    'ec2-user@11.11.11.11',
                ],
                id='Public: connect to public IP',
            ),
            pytest.param(
                get_describe_public_instance_response(),
                None,
                None,
                [
                    "ec2-instance-connect",
                    "ssh",
                    "--instance-id",
                    "i-123",
                    "--private-key-file",
                    "/tmp/ssh-file",
                    "--instance-ip",
                    "10.0.0.1",
                    "--connection-type",
                    "direct",
                ],
                [
                    'ssh',
                    '-o',
                    'ServerAliveInterval=5',
                    '-p',
                    '22',
                    '-i',
                    '/tmp/ssh-file',
                    'ec2-user@10.0.0.1',
                ],
                id='Public: connect to provided ip address',
            ),
            pytest.param(
                get_describe_public_instance_response(),
                None,
                None,
                [
                    "ec2-instance-connect",
                    "ssh",
                    "--instance-id",
                    "i-123",
                    "--private-key-file",
                    "/tmp/ssh-file",
                    "--connection-type",
                    "direct",
                ],
                [
                    'ssh',
                    '-o',
                    'ServerAliveInterval=5',
                    '-p',
                    '22',
                    '-i',
                    '/tmp/ssh-file',
                    'ec2-user@11.11.11.11',
                ],
                id='Public: use public when connection-type direct',
            ),
            pytest.param(
                get_describe_private_instance_response(),
                None,
                None,
                [
                    "ec2-instance-connect",
                    "ssh",
                    "--instance-id",
                    "i-123",
                    "--private-key-file",
                    "/tmp/ssh-file",
                    "--connection-type",
                    "direct",
                ],
                [
                    'ssh',
                    '-o',
                    'ServerAliveInterval=5',
                    '-p',
                    '22',
                    '-i',
                    '/tmp/ssh-file',
                    'ec2-user@10.0.0.0',
                ],
                id='Public: use private ip since no public ip when connection-type direct',
            ),
            pytest.param(
                get_describe_ipv6_instance_response(),
                None,
                None,
                [
                    "ec2-instance-connect",
                    "ssh",
                    "--instance-id",
                    "i-123",
                    "--private-key-file",
                    "/tmp/ssh-file",
                    "--connection-type",
                    "direct",
                ],
                [
                    'ssh',
                    '-o',
                    'ServerAliveInterval=5',
                    '-p',
                    '22',
                    '-i',
                    '/tmp/ssh-file',
                    'ec2-user@2600:1f10:4f8e:db01:73f5:6b9d:c0da:1c27',
                ],
                id='Public: IPv6: use ipv6 when connection-type direct and no IPv4 address',
            ),
        ],
    )
    @mock.patch("shutil.which")
    @mock.patch("subprocess.call")
    @mock.patch.object(sys, 'argv', ['aws'])
    def test_successful_ssh_for_instance(
        self,
        mock_subproc_call,
        mock_which,
        cli_runner,
        request_params_for_describe_instance,
        describe_instance_response,
        describe_eice_response,
        request_params_for_describe_eice,
        cli_input,
        expected_ssh_command,
    ):
        mock_which.return_value = "ssh"
        mock_subproc_call.return_value = 0
        cli_runner.add_response(HTTPResponse(body=describe_instance_response))
        if describe_eice_response:
            cli_runner.add_response(HTTPResponse(body=describe_eice_response))

        result = cli_runner.run(cli_input)

        assert 0 == result.rc
        assert mock_subproc_call.called
        requests = [
            AWSRequest(
                service_name='ec2',
                operation_name='DescribeInstances',
                params=request_params_for_describe_instance,
            ),
        ]
        if describe_eice_response:
            requests += [
                AWSRequest(
                    service_name='ec2',
                    operation_name='DescribeInstanceConnectEndpoints',
                    params=request_params_for_describe_eice,
                ),
            ]
        assert result.aws_requests == requests
        mock_which.assert_called_once_with('ssh')
        mock_subproc_call.assert_called_once_with(expected_ssh_command)

    @mock.patch("shutil.which")
    @mock.patch("subprocess.call")
    @mock.patch.object(sys, 'argv', ['aws.cmd'])
    def test_valid_input_without_ssh_key(
        self,
        mock_subproc_call,
        mock_which,
        cli_runner,
        describe_eice_response,
        describe_private_instance_response,
        send_ssh_public_key_response,
    ):
        mock_which.return_value = '/some/path/to/ssh'
        mock_subproc_call.return_value = 0
        cli_runner.add_response(
            HTTPResponse(body=describe_private_instance_response)
        )
        cli_runner.add_response(HTTPResponse(body=describe_eice_response))
        cli_runner.add_response(
            HTTPResponse(body=send_ssh_public_key_response)
        )

        result = cli_runner.run(
            [
                "ec2-instance-connect",
                "ssh",
                "--instance-id",
                "i-04997c129f59becde",
            ]
        )

        assert 0 == result.rc
        assert mock_subproc_call.called
        assert result.aws_requests == [
            AWSRequest(
                service_name='ec2',
                operation_name='DescribeInstances',
                params={'InstanceIds': ['i-04997c129f59becde']},
            ),
            AWSRequest(
                service_name='ec2',
                operation_name='DescribeInstanceConnectEndpoints',
                params=mock.ANY,
            ),
            AWSRequest(
                service_name='ec2-instance-connect',
                operation_name='SendSSHPublicKey',
                params=mock.ANY,
            ),
        ]
        mock_which.assert_called_once_with('ssh')
        mock_subproc_call.assert_called_once_with(
            [
                '/some/path/to/ssh',
                '-o',
                'ServerAliveInterval=5',
                '-p',
                '22',
                '-i',
                mock.ANY,
                '-o',
                'ProxyCommand=aws.cmd ec2-instance-connect open-tunnel --instance-id i-04997c129f59becde '
                '--private-ip-address 10.0.0.0 --remote-port 22 '
                '--instance-connect-endpoint-id eice-123 --instance-connect-endpoint-dns-name dns.com',
                'ec2-user@10.0.0.0',
            ]
        )

    def test_no_ip_on_instance(
        self, cli_runner, describe_no_ip_instance_response
    ):
        cli_runner.add_response(
            HTTPResponse(body=describe_no_ip_instance_response)
        )

        result = cli_runner.run(
            [
                "ec2-instance-connect",
                "ssh",
                "--instance-id",
                "i-04997c129f59becde",
                "--private-key-file",
                "/tmp/ssh-file",
            ]
        )

        assert 252 == result.rc
        assert (
            'Unable to find any IP address on the instance to connect to.'
            in result.stderr
        )

    def test_command_fails_when_eice_connection_type_and_no_private_ip(
        self, cli_runner, describe_no_ip_instance_response
    ):
        cli_runner.add_response(
            HTTPResponse(body=describe_no_ip_instance_response)
        )

        result = cli_runner.run(
            [
                "ec2-instance-connect",
                "ssh",
                "--instance-id",
                "i-04997c129f59becde",
                "--private-key-file",
                "/tmp/ssh-file",
                "--connection-type",
                "eice",
            ]
        )

        assert 252 == result.rc
        assert (
            'Unable to find any IP address on the instance to connect to.'
            in result.stderr
        )

    @pytest.mark.parametrize(
        "cli_input,expected_error_code,expected",
        [
            pytest.param(
                [
                    "ec2-instance-connect",
                    "ssh",
                ],
                252,
                "the following arguments are required: --instance-id",
                id="Failure: Customer must provide instance-id",
            ),
            pytest.param(
                [
                    "ec2-instance-connect",
                    "ssh",
                    "--instance-id",
                    "i-12345",
                    "--connection-type",
                    "test",
                ],
                252,
                "argument --connection-type: Found invalid choice 'test'",
                id='Failure: Customer must provide connection-type when IP defined',
            ),
            pytest.param(
                [
                    "ec2-instance-connect",
                    "ssh",
                    "--instance-id",
                    "i-12345",
                    "--connection-type",
                    "direct",
                    "--eice-options",
                    "endpointId=eice-12345",
                ],
                252,
                "eice-options can't be specified when connection type is direct.",
                id='Failure: Customer can not use connection-type direct with eice-id',
            ),
            pytest.param(
                [
                    "ec2-instance-connect",
                    "ssh",
                    "--instance-id",
                    "i-12345",
                    "--connection-type",
                    "direct",
                    "--eice-options",
                    "dnsName=test.com",
                ],
                252,
                "eice-options can't be specified when connection type is direct.",
                id='Failure: Customer can not use connection-type direct with eice-dns-name',
            ),
            pytest.param(
                [
                    "ec2-instance-connect",
                    "ssh",
                    "--instance-id",
                    "i-12345",
                    "--connection-type",
                    "direct",
                    "--eice-options",
                    "maxTunnelDuration=12",
                ],
                252,
                "eice-options can't be specified when connection type is direct.",
                id='Failure: Customer can not use connection-type direct with maxTunnelDuration',
            ),
            pytest.param(
                [
                    "ec2-instance-connect",
                    "ssh",
                    "--instance-id",
                    "i-12345",
                    "--eice-options",
                    "dnsName=test",
                ],
                252,
                "When specifying dnsName, you must specify endpointId.",
                id='Failure: Customer must define eice-id when eice-dns provided',
            ),
            pytest.param(
                [
                    "ec2-instance-connect",
                    "ssh",
                    "--instance-id",
                    "i-12345",
                    "--ssh-port",
                    "ab",
                ],
                255,
                "invalid literal for int() with base 10",
                id='Failure: port must be int',
            ),
            pytest.param(
                [
                    "ec2-instance-connect",
                    "ssh",
                    "--instance-id",
                    "i-12345",
                    "--eice-options",
                    "maxTunnelDuration=ab",
                ],
                255,
                'invalid literal for int() with base 10',
                id='Failure: maxTunnelDuration must be int',
            ),
            pytest.param(
                [
                    "ec2-instance-connect",
                    "ssh",
                    "--instance-id",
                    "i-12345",
                    "--instance-ip",
                    "10.0.0.1",
                ],
                252,
                'When specifying instance-ip, you must specify connection-type.',
                id='Failure: Customer must define connection-type when ip define instead of Instance ID',
            ),
            pytest.param(
                ["ec2-instance-connect", "ssh", "--instance-id", "a-12345"],
                252,
                'The specified instance ID is invalid.',
                id='Failure: Customer provide valid Instance ID',
            ),
            pytest.param(
                ["ec2-instance-connect", "ssh", "--instance-id", "i-12+3:4;5"],
                252,
                'The specified instance ID is invalid.',
                id='Failure: Customer provide valid Instance ID without special characters',
            ),
            pytest.param(
                [
                    "ec2-instance-connect",
                    "ssh",
                    "--instance-id",
                    "i-12345",
                    "--eice-options",
                    "endpointId=aeice-12345",
                ],
                252,
                'The specified endpointId is invalid.',
                id='Failure: Customer must define valid EICE ID',
            ),
            pytest.param(
                [
                    "ec2-instance-connect",
                    "ssh",
                    "--instance-id",
                    "i-12345",
                    "--eice-options",
                    "endpointId=eice-12;3:4+5-",
                ],
                252,
                'The specified endpointId is invalid.',
                id='Failure: Customer must define valid EICE ID without special characters',
            ),
            pytest.param(
                [
                    "ec2-instance-connect",
                    "ssh",
                    "--instance-id",
                    "i-12345",
                    "--eice-options",
                    "endpointId=eice-12345,dnsName=dns.domain.+com",
                ],
                252,
                'The specified dnsName is invalid.',
                id='Failure: Customer must define valid EICE dnsName without special characters',
            ),
            pytest.param(
                [
                    "ec2-instance-connect",
                    "ssh",
                    "--instance-id",
                    "i-12345",
                    "--eice-options",
                    "test=123",
                ],
                252,
                'Unknown parameter in input',
                id="Failure: Customer must define allowed eice options",
            ),
        ],
    )
    def test_command_fails_when_invalid_input(
        self, cli_runner, cli_input, expected_error_code, expected
    ):
        result = cli_runner.run(cli_input)

        assert expected_error_code == result.rc
        assert expected in result.stderr

    @pytest.mark.parametrize("duration", ["-1", "0", "3601"])
    def test_command_fails_when_using_invalid_max_tunnel_duration(
        self, cli_runner, duration
    ):
        cmdline = [
            "ec2-instance-connect",
            "ssh",
            "--instance-id",
            "i-123",
            "--eice-options",
            f"maxTunnelDuration={duration}",
        ]
        result = cli_runner.run(cmdline)

        assert 252 == result.rc
        assert (
            "Invalid value specified for maxTunnelDuration. Value must be greater than 1 and less than 3600."
            in result.stderr
        )

    @mock.patch("shutil.which")
    @mock.patch.object(sys, 'argv', ['aws'])
    def test_command_error_when_ssh_not_available(
        self,
        mock_which,
        cli_runner,
        describe_public_instance_response,
        request_params_for_describe_instance,
    ):
        mock_which.return_value = None
        cli_runner.add_response(
            HTTPResponse(body=describe_public_instance_response)
        )

        result = cli_runner.run(
            [
                "ec2-instance-connect",
                "ssh",
                "--instance-id",
                "i-123",
                "--private-key-file",
                "/tmp/ssh-file",
            ]
        )

        assert 253 == result.rc
        assert 'SSH not available.' in result.stderr
        assert result.aws_requests == [
            AWSRequest(
                service_name='ec2',
                operation_name='DescribeInstances',
                params=request_params_for_describe_instance,
            ),
        ]



# Copyright 2020 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from prompt_toolkit.application import Application
from prompt_toolkit.layout import Layout

from awscli.customizations.wizard.app import WizardTraverser
from awscli.customizations.wizard.ui.utils import (
    get_ui_control_by_buffer_name,
    move_to_previous_prompt,
)
from awscli.testutils import mock, unittest


class TestGetUIControlByName(unittest.TestCase):
    def test_get_ui_control_by_name(self):
        layout = mock.Mock(Layout)
        desired_ui_control = mock.Mock()
        desired_ui_control.buffer.name = 'desired-buffer'
        other_ui_control = mock.Mock()
        other_ui_control.buffer.name = 'different-name'
        layout.find_all_controls.return_value = [
            object(),  # Does not have a buffer property
            other_ui_control,
            desired_ui_control,
        ]
        self.assertIs(
            get_ui_control_by_buffer_name(layout, 'desired-buffer'),
            desired_ui_control,
        )

    def test_raises_error_when_no_matching_ui_control_found(self):
        layout = mock.Mock(Layout)
        layout.find_all_controls.return_value = []
        with self.assertRaises(ValueError):
            get_ui_control_by_buffer_name(layout, 'buffer-does-not-exist')


class TestMoveToPreviousPrompt(unittest.TestCase):
    def test_move_to_previous_prompt(self):
        app = mock.Mock(Application)
        app.layout = mock.Mock(Layout)
        previous_control = mock.Mock()
        previous_control.buffer.name = 'previous'
        app.layout.find_all_controls.return_value = [
            previous_control,
        ]
        app.traverser = mock.Mock(WizardTraverser)
        app.traverser.previous_prompt.return_value = 'previous'

        move_to_previous_prompt(app)
        app.layout.focus.assert_called_with(previous_control)



# Copyright 2020 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from prompt_toolkit.buffer import Buffer
from prompt_toolkit.completion import Completer, DummyCompleter
from prompt_toolkit.layout import Window
from prompt_toolkit.widgets import SearchToolbar

from awscli.autoprompt.factory import (
    CLIPromptBuffer,
    PromptToolkitFactory,
    PromptToolkitKeyBindings,
)
from awscli.autoprompt.history import HistoryCompleter
from awscli.testutils import mock, unittest


class FakeCLIPromptBuffer(CLIPromptBuffer):
    def start_completion(self):
        pass


class TestPromptToolkitFactory(unittest.TestCase):
    def setUp(self):
        self.factory = PromptToolkitFactory(completer=DummyCompleter())

    def dummy_callback(self, *args, **kwargs):
        return

    def test_can_create_input_buffer(self):
        buffer = self.factory.create_input_buffer()
        self.assertEqual(buffer.name, 'input_buffer')

    def test_can_create_input_buffer_with_callback(self):
        buffer = self.factory.create_input_buffer(self.dummy_callback)
        self.assertIsNotNone(buffer.on_text_changed)

    def test_can_create_doc_buffer(self):
        buffer = self.factory.create_doc_buffer()
        self.assertEqual(buffer.name, 'doc_buffer')

    def test_can_create_input_buffer_container(self):
        buffer = mock.Mock(spec=Buffer)
        container = self.factory.create_input_buffer_container(buffer)
        self.assertIsNotNone(container.content)

    def test_can_create_searchable_window(self):
        buffer = mock.Mock(spec=Buffer)
        container = self.factory.create_searchable_window('foo window', buffer)
        self.assertIsNotNone(container.children)

    def test_can_create_bottom_panel(self):
        buffer = mock.Mock(spec=Buffer)
        foo = self.factory.create_searchable_window('foo window', buffer)
        bar = self.factory.create_searchable_window('bar window', buffer)
        panel = self.factory.create_bottom_panel(foo, bar)
        self.assertTrue(len(panel.children), 3)

    def test_can_create_layout(self):
        layout = self.factory.create_layout()
        self.assertIsNotNone(layout.container)

    def test_can_create_layout_with_input_buffer_callback_specified(self):
        layout = self.factory.create_layout(
            on_input_buffer_text_changed=self.dummy_callback
        )
        self.assertIsNotNone(layout.container)

    def test_can_create_layout_with_input_buffer_container_specified(self):
        layout = self.factory.create_layout(input_buffer_container=Window())
        self.assertIsNotNone(layout.container)

    def test_can_create_layout_with_doc_window_specified(self):
        layout = self.factory.create_layout(doc_window=Window())
        self.assertIsNotNone(layout.container)

    def test_can_create_key_bindings(self):
        key_bindings = self.factory.create_key_bindings()
        self.assertIsInstance(key_bindings, PromptToolkitKeyBindings)


class TestCLIPromptBuffer(unittest.TestCase):
    def setUp(self):
        self.buffer = FakeCLIPromptBuffer()

    def test_history_mode_switching(self):
        self.buffer.switch_history_mode()
        self.assertIsInstance(self.buffer.completer, HistoryCompleter)
        self.buffer.switch_history_mode()
        self.assertIsInstance(self.buffer.completer, Completer)



# Copyright 2018 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the 'License'). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the 'license' file accompanying this file. This file is
# distributed on an 'AS IS' BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from decimal import Decimal

from awscli.customizations.dynamodb.types import (
    Binary,
    TypeDeserializer,
    TypeSerializer,
)
from awscli.testutils import unittest


class TestBinary(unittest.TestCase):
    def test_bytes_input(self):
        data = Binary(b'\x01')
        self.assertEqual(b'\x01', data)
        self.assertEqual(b'\x01', data.value)

    def test_non_ascii_bytes_input(self):
        # Binary data that is out of ASCII range
        data = Binary(b'\x88')
        self.assertEqual(b'\x88', data)
        self.assertEqual(b'\x88', data.value)

    def test_bytearray_input(self):
        data = Binary(bytearray([1]))
        self.assertEqual(b'\x01', data)
        self.assertEqual(b'\x01', data.value)

    def test_unicode_throws_error(self):
        with self.assertRaises(TypeError):
            Binary('\u00e9')

    def test_integer_throws_error(self):
        with self.assertRaises(TypeError):
            Binary(1)

    def test_not_equal(self):
        self.assertTrue(Binary(b'\x01') != b'\x02')

    def test_str(self):
        self.assertEqual(Binary(b'\x01').__str__(), b'\x01')

    def test_repr(self):
        self.assertIn('Binary', repr(Binary(b'1')))


class TestSerializer(unittest.TestCase):
    def setUp(self):
        self.serializer = TypeSerializer()

    def test_serialize_unsupported_type(self):
        with self.assertRaisesRegex(TypeError, 'Unsupported type'):
            self.serializer.serialize(object())

    def test_serialize_null(self):
        self.assertEqual(self.serializer.serialize(None), {'NULL': True})

    def test_serialize_boolean(self):
        self.assertEqual(self.serializer.serialize(False), {'BOOL': False})

    def test_serialize_integer(self):
        self.assertEqual(self.serializer.serialize(1), {'N': '1'})

    def test_serialize_decimal(self):
        self.assertEqual(
            self.serializer.serialize(Decimal('1.25')), {'N': '1.25'}
        )

    def test_serialize_float_error(self):
        with self.assertRaisesRegex(
            TypeError,
            'Float types are not supported. Use Decimal types instead',
        ):
            self.serializer.serialize(1.25)

    def test_serialize_NaN_error(self):
        with self.assertRaisesRegex(
            TypeError, 'Infinity and NaN not supported'
        ):
            self.serializer.serialize(Decimal('NaN'))

    def test_serialize_string(self):
        self.assertEqual(self.serializer.serialize('foo'), {'S': 'foo'})

    def test_serialize_binary(self):
        self.assertEqual(
            self.serializer.serialize(Binary(b'\x01')), {'B': b'\x01'}
        )

    def test_serialize_bytearray(self):
        self.assertEqual(
            self.serializer.serialize(bytearray([1])), {'B': b'\x01'}
        )

    def test_serialize_bytes(self):
        self.assertEqual(self.serializer.serialize(b'\x01'), {'B': b'\x01'})

    def test_serialize_number_set(self):
        serialized_value = self.serializer.serialize(set([1, 2, 3]))
        self.assertEqual(len(serialized_value), 1)
        self.assertIn('NS', serialized_value)
        self.assertCountEqual(serialized_value['NS'], ['1', '2', '3'])

    def test_serialize_string_set(self):
        serialized_value = self.serializer.serialize(set(['foo', 'bar']))
        self.assertEqual(len(serialized_value), 1)
        self.assertIn('SS', serialized_value)
        self.assertCountEqual(serialized_value['SS'], ['foo', 'bar'])

    def test_serialize_binary_set(self):
        serialized_value = self.serializer.serialize(
            set([Binary(b'\x01'), Binary(b'\x02')])
        )
        self.assertEqual(len(serialized_value), 1)
        self.assertIn('BS', serialized_value)
        self.assertCountEqual(serialized_value['BS'], [b'\x01', b'\x02'])

    def test_serialize_list(self):
        serialized_value = self.serializer.serialize(['foo', 1, [1]])
        self.assertEqual(len(serialized_value), 1)
        self.assertIn('L', serialized_value)
        self.assertCountEqual(
            serialized_value['L'],
            [{'S': 'foo'}, {'N': '1'}, {'L': [{'N': '1'}]}],
        )

    def test_serialize_map(self):
        serialized_value = self.serializer.serialize(
            {'foo': 'bar', 'baz': {'biz': 1}}
        )
        self.assertEqual(
            serialized_value,
            {'M': {'foo': {'S': 'bar'}, 'baz': {'M': {'biz': {'N': '1'}}}}},
        )


class TestDeserializer(unittest.TestCase):
    def setUp(self):
        self.deserializer = TypeDeserializer()

    def test_deserialize_invalid_type(self):
        with self.assertRaisesRegex(TypeError, 'FOO is not supported'):
            self.deserializer.deserialize({'FOO': 'bar'})

    def test_deserialize_empty_structure(self):
        with self.assertRaisesRegex(TypeError, 'Value must be a nonempty'):
            self.assertEqual(self.deserializer.deserialize({}), {})

    def test_deserialize_null(self):
        self.assertEqual(self.deserializer.deserialize({"NULL": True}), None)

    def test_deserialize_boolean(self):
        self.assertEqual(self.deserializer.deserialize({"BOOL": False}), False)

    def test_deserialize_integer(self):
        self.assertEqual(
            self.deserializer.deserialize({'N': '1'}), Decimal('1')
        )

    def test_deserialize_decimal(self):
        self.assertEqual(
            self.deserializer.deserialize({'N': '1.25'}), Decimal('1.25')
        )

    def test_deserialize_string(self):
        self.assertEqual(self.deserializer.deserialize({'S': 'foo'}), 'foo')

    def test_deserialize_binary(self):
        self.assertEqual(
            self.deserializer.deserialize({'B': b'\x00'}), Binary(b'\x00')
        )

    def test_deserialize_number_set(self):
        self.assertEqual(
            self.deserializer.deserialize({'NS': ['1', '1.25']}),
            set([Decimal('1'), Decimal('1.25')]),
        )

    def test_deserialize_string_set(self):
        self.assertEqual(
            self.deserializer.deserialize({'SS': ['foo', 'bar']}),
            set(['foo', 'bar']),
        )

    def test_deserialize_binary_set(self):
        self.assertEqual(
            self.deserializer.deserialize({'BS': [b'\x00', b'\x01']}),
            set([Binary(b'\x00'), Binary(b'\x01')]),
        )

    def test_deserialize_list(self):
        self.assertEqual(
            self.deserializer.deserialize(
                {'L': [{'N': '1'}, {'S': 'foo'}, {'L': [{'N': '1.25'}]}]}
            ),
            [Decimal('1'), 'foo', [Decimal('1.25')]],
        )

    def test_deserialize_map(self):
        self.assertEqual(
            self.deserializer.deserialize(
                {
                    'M': {
                        'foo': {'S': 'mystring'},
                        'bar': {'M': {'baz': {'N': '1'}}},
                    }
                }
            ),
            {'foo': 'mystring', 'bar': {'baz': Decimal('1')}},
        )



#!/usr/bin/env python
# Copyright 2013-2014 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from awscli.testutils import BaseAWSCommandParamsTest


class TestSendEmail(BaseAWSCommandParamsTest):
    prefix = 'ses send-email'

    def test_plain_text(self):
        args = (
            ' --subject This_is_a_test --from foo@bar.com'
            ' --to fie@baz.com --text This_is_the_message'
        )
        args_list = (self.prefix + args).split()
        result = {
            'Source': 'foo@bar.com',
            'Destination': {'ToAddresses': ['fie@baz.com']},
            'Message': {
                'Body': {'Text': {'Data': 'This_is_the_message'}},
                'Subject': {'Data': 'This_is_a_test'},
            },
        }
        self.assert_params_for_cmd(args_list, result)

    def test_plain_text_multiple_to(self):
        args = (
            ' --subject This_is_a_test --from foo@bar.com'
            ' --to fie1@baz.com fie2@baz.com --text This_is_the_message'
        )
        args_list = (self.prefix + args).split()
        result = {
            'Source': 'foo@bar.com',
            'Destination': {'ToAddresses': ['fie1@baz.com', 'fie2@baz.com']},
            'Message': {
                'Body': {'Text': {'Data': 'This_is_the_message'}},
                'Subject': {'Data': 'This_is_a_test'},
            },
        }

        self.assert_params_for_cmd(args_list, result)

    def test_plain_text_multiple_cc(self):
        args = (
            ' --subject This_is_a_test --from foo@bar.com'
            ' --to fie1@baz.com fie2@baz.com --text This_is_the_message'
            ' --cc fie3@baz.com fie4@baz.com'
        )
        args_list = (self.prefix + args).split()
        result = {
            'Source': 'foo@bar.com',
            'Destination': {
                'CcAddresses': ['fie3@baz.com', 'fie4@baz.com'],
                'ToAddresses': ['fie1@baz.com', 'fie2@baz.com'],
            },
            'Message': {
                'Body': {'Text': {'Data': 'This_is_the_message'}},
                'Subject': {'Data': 'This_is_a_test'},
            },
        }

        self.assert_params_for_cmd(args_list, result)

    def test_plain_text_multiple_bcc(self):
        args = (
            ' --subject This_is_a_test --from foo@bar.com'
            ' --to fie1@baz.com fie2@baz.com --text This_is_the_message'
            ' --cc fie3@baz.com fie4@baz.com'
            ' --bcc fie5@baz.com fie6@baz.com'
        )
        args_list = (self.prefix + args).split()

        result = {
            'Source': 'foo@bar.com',
            'Destination': {
                'BccAddresses': ['fie5@baz.com', 'fie6@baz.com'],
                'CcAddresses': ['fie3@baz.com', 'fie4@baz.com'],
                'ToAddresses': ['fie1@baz.com', 'fie2@baz.com'],
            },
            'Message': {
                'Body': {'Text': {'Data': 'This_is_the_message'}},
                'Subject': {'Data': 'This_is_a_test'},
            },
        }

        self.assert_params_for_cmd(args_list, result)

    def test_html_text(self):
        args = (
            ' --subject This_is_a_test --from foo@bar.com'
            ' --to fie@baz.com --html This_is_the_html_message'
        )
        args_list = (self.prefix + args).split()
        result = {
            'Source': 'foo@bar.com',
            'Destination': {'ToAddresses': ['fie@baz.com']},
            'Message': {
                'Subject': {'Data': 'This_is_a_test'},
                'Body': {'Html': {'Data': 'This_is_the_html_message'}},
            },
        }

        self.assert_params_for_cmd(args_list, result)

    def test_html_both(self):
        args = (
            ' --subject This_is_a_test --from foo@bar.com'
            ' --to fie@baz.com --html This_is_the_html_message'
            ' --text This_is_the_text_message'
        )
        args_list = (self.prefix + args).split()
        result = {
            'Source': 'foo@bar.com',
            'Destination': {'ToAddresses': ['fie@baz.com']},
            'Message': {
                'Subject': {'Data': 'This_is_a_test'},
                'Body': {
                    'Text': {'Data': 'This_is_the_text_message'},
                    'Html': {'Data': 'This_is_the_html_message'},
                },
            },
        }
        self.assert_params_for_cmd(args_list, result)

    def test_using_json(self):
        args = (
            ' --message {"Subject":{"Data":"This_is_a_test"},'
            '"Body":{"Text":{"Data":"This_is_the_message"}}}'
            ' --from foo@bar.com'
            ' --destination {"ToAddresses":["fie@baz.com"]}'
        )
        args_list = (self.prefix + args).split()
        result = {
            'Destination': {'ToAddresses': ['fie@baz.com']},
            'Message': {
                'Subject': {'Data': 'This_is_a_test'},
                'Body': {'Text': {'Data': 'This_is_the_message'}},
            },
            'Source': 'foo@bar.com',
        }

        self.assert_params_for_cmd(args_list, result)

    def test_both_destination_and_to(self):
        args = (
            ' --message {"Subject":{"Data":"This_is_a_test"},'
            '"Body":{"Text":{"Data":"This_is_the_message"}}}'
            ' --from foo@bar.com'
            ' --destination {"ToAddresses":["fie@baz.com"]}'
            ' --to fie2@baz.com'
        )
        args_list = (self.prefix + args).split()
        self.run_cmd(args_list, expected_rc=252)

    def test_both_message_and_text(self):
        args = (
            ' --message {"Subject":{"Data":"This_is_a_test"},'
            '"Body":{"Text":{"Data":"This_is_the_message"}}}'
            ' --from foo@bar.com'
            ' --destination {"ToAddresses":["fie@baz.com"]}'
            ' --text This_is_another_body'
        )
        args_list = (self.prefix + args).split()
        self.run_cmd(args_list, expected_rc=252)



# Copyright 2015 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

from argparse import Namespace

from awscli.customizations.codedeploy.register import Register
from awscli.customizations.codedeploy.utils import MAX_TAGS_PER_INSTANCE
from awscli.customizations.exceptions import (
    ConfigurationError,
    ParamValidationError,
)
from awscli.testutils import mock, unittest


class TestRegister(unittest.TestCase):
    def setUp(self):
        self.path = '/AWS/CodeDeploy/'
        self.instance_name = 'instance-name'
        self.tags = [{'Key': 'k1', 'Value': 'v1'}]
        self.iam_user_arn = 'arn:aws:iam::012345678912:user/instance-name'
        self.access_key_id = 'ACCESSKEYID'
        self.secret_access_key = 'SECRETACCESSKEY'
        self.region = 'us-east-1'
        self.policy_name = 'codedeploy-agent'
        self.policy_document = (
            '{\n'
            '    "Version": "2012-10-17",\n'
            '    "Statement": [ {\n'
            '        "Action": [ "s3:Get*", "s3:List*" ],\n'
            '        "Effect": "Allow",\n'
            '        "Resource": "*"\n'
            '    } ]\n'
            '}'
        )
        self.config_file = 'codedeploy.onpremises.yml'
        self.endpoint_url = 'https://codedeploy.aws.amazon.com'

        self.args = Namespace()
        self.args.instance_name = self.instance_name
        self.args.tags = None
        self.args.iam_user_arn = None

        self.globals = Namespace()
        self.globals.region = self.region
        self.globals.endpoint_url = self.endpoint_url
        self.globals.verify_ssl = False

        self.os_open_patcher = mock.patch(
            'awscli.customizations.codedeploy.register.os.open',
            return_value=3,
        )
        self.os_open = self.os_open_patcher.start()

        self.os_fdopen_patcher = mock.patch(
            'awscli.customizations.codedeploy.register.os.fdopen',
            mock.mock_open(),
        )
        self.os_fdopen = self.os_fdopen_patcher.start()

        self.os_chmod_patcher = mock.patch(
            'awscli.customizations.codedeploy.register.os.chmod'
        )
        self.os_chmod = self.os_chmod_patcher.start()

        self.codedeploy = mock.MagicMock()

        self.iam = mock.MagicMock()
        self.iam.create_user.return_value = {
            'User': {'Arn': self.iam_user_arn}
        }
        self.iam.create_access_key.return_value = {
            'AccessKey': {
                'AccessKeyId': self.access_key_id,
                'SecretAccessKey': self.secret_access_key,
            }
        }

        self.session = mock.MagicMock()
        self.session.create_client.side_effect = [self.codedeploy, self.iam]
        self.register = Register(self.session)

    def tearDown(self):
        self.os_open_patcher.stop()
        self.os_fdopen_patcher.stop()
        self.os_chmod_patcher.stop()

    def test_register_throws_on_invalid_region(self):
        self.globals.region = None
        self.session.get_config_variable.return_value = None
        error_msg = 'Region not specified.'
        with self.assertRaisesRegex(ConfigurationError, error_msg):
            self.register._run_main(self.args, self.globals)

    def test_register_throws_on_invalid_instance_name(self):
        self.args.instance_name = 'invalid%@^&%#&'
        error_msg = 'Instance name contains invalid characters.'
        with self.assertRaisesRegex(ParamValidationError, error_msg):
            self.register._run_main(self.args, self.globals)

    def test_register_throws_on_invalid_tags(self):
        self.args.tags = [
            {'Key': 'k' + str(x), 'Value': 'v' + str(x)} for x in range(11)
        ]
        with self.assertRaisesRegex(
            ParamValidationError,
            f'Instances can only have a maximum of {MAX_TAGS_PER_INSTANCE} tags.',
        ):
            self.register._run_main(self.args, self.globals)

    def test_register_throws_on_invalid_iam_user_arn(self):
        self.args.iam_user_arn = 'invalid%@^&%#&'
        error_msg = 'Invalid IAM user ARN.'
        with self.assertRaisesRegex(ParamValidationError, error_msg):
            self.register._run_main(self.args, self.globals)

    def test_register_creates_clients(self):
        self.register._run_main(self.args, self.globals)
        self.session.create_client.assert_has_calls(
            [
                mock.call(
                    'codedeploy',
                    region_name=self.region,
                    endpoint_url=self.endpoint_url,
                    verify=self.globals.verify_ssl,
                ),
                mock.call('iam', region_name=self.region),
            ]
        )

    def test_register_with_no_iam_user_arn(self):
        self.args.iam_user_arn = None
        self.register._run_main(self.args, self.globals)
        self.register.iam.create_user.assert_called_with(
            Path=self.path, UserName=self.instance_name
        )
        self.assertIn('iam_user_arn', self.args)
        self.assertEqual(self.iam_user_arn, self.args.iam_user_arn)
        self.register.iam.create_access_key.assert_called_with(
            UserName=self.instance_name
        )
        self.assertIn('access_key_id', self.args)
        self.assertEqual(self.access_key_id, self.args.access_key_id)
        self.assertIn('secret_access_key', self.args)
        self.assertEqual(self.secret_access_key, self.args.secret_access_key)
        self.register.iam.put_user_policy.assert_called_with(
            UserName=self.instance_name,
            PolicyName=self.policy_name,
            PolicyDocument=self.policy_document,
        )
        self.assertIn('policy_name', self.args)
        self.assertEqual(self.policy_name, self.args.policy_name)
        self.assertIn('policy_document', self.args)
        self.assertEqual(self.policy_document, self.args.policy_document)
        self.os_open.assert_called_with(
            self.config_file,
            mock.ANY,
            0o600,
        )
        self.os_fdopen.assert_called_with(3, 'w')
        self.os_fdopen().write.assert_called_with(
            '---\n'
            f'region: {self.region}\n'
            f'iam_user_arn: {self.iam_user_arn}\n'
            f'aws_access_key_id: {self.access_key_id}\n'
            f'aws_secret_access_key: {self.secret_access_key}\n'
        )
        self.register.codedeploy.register_on_premises_instance.assert_called_with(
            instanceName=self.instance_name, iamUserArn=self.iam_user_arn
        )

    def test_register_with_iam_user_arn(self):
        self.args.iam_user_arn = self.iam_user_arn
        self.register._run_main(self.args, self.globals)
        self.assertFalse(self.register.iam.create_user.called)
        self.assertFalse(self.register.iam.create_access_key.called)
        self.assertFalse(self.register.iam.put_user_policy.called)
        self.assertFalse(self.os_open.called)
        self.register.codedeploy.register_on_premises_instance.assert_called_with(
            instanceName=self.instance_name, iamUserArn=self.iam_user_arn
        )

    def test_register_with_no_tags(self):
        self.args.tags = None
        self.register._run_main(self.args, self.globals)
        self.register.codedeploy.register_on_premises_instance.assert_called_with(
            instanceName=self.instance_name, iamUserArn=self.iam_user_arn
        )
        self.assertFalse(
            self.register.codedeploy.add_tags_to_on_premises_instances.called
        )

    def test_register_with_tags(self):
        self.args.tags = self.tags
        self.register._run_main(self.args, self.globals)
        self.register.codedeploy.register_on_premises_instance.assert_called_with(
            instanceName=self.instance_name, iamUserArn=self.iam_user_arn
        )
        self.register.codedeploy.add_tags_to_on_premises_instances.assert_called_with(
            tags=self.tags, instanceNames=[self.instance_name]
        )

    def test_create_config_raises_runtime_error_on_open_failure(self):
        self.args.iam_user_arn = None
        self.os_open.side_effect = OSError('permission denied')
        with self.assertRaisesRegex(
            RuntimeError, 'Failed to create config file'
        ):
            self.register._create_config(self.args)

    def test_create_config_raises_runtime_error_on_chmod_failure(self):
        self.args.iam_user_arn = None
        self.os_chmod.side_effect = OSError('permission denied')
        with self.assertRaisesRegex(
            RuntimeError, 'Failed to create config file'
        ):
            self.register._create_config(self.args)

    def test_create_config_uses_restricted_permissions(self):
        self.args.iam_user_arn = None
        self.register._run_main(self.args, self.globals)
        self.os_open.assert_called_with(
            self.config_file,
            mock.ANY,
            0o600,
        )

    def test_create_config_chmods_existing_file(self):
        self.args.iam_user_arn = None
        self.register._run_main(self.args, self.globals)
        self.os_chmod.assert_called_with(self.config_file, 0o600)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# Copyright 2015 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import os

from awscli.testutils import BaseAWSCommandParamsTest, FileCreator


class TestOutFileQueryArguments(BaseAWSCommandParamsTest):
    def setUp(self):
        self.files = FileCreator()
        super(TestOutFileQueryArguments, self).setUp()

    def tearDown(self):
        self.files.remove_all()
        super(TestOutFileQueryArguments, self).tearDown()

    def test_saves_cert_to_file_for_create_certificate_from_csr(self):
        self.parsed_response = {
            'certificatePem': 'cert...',
            'ResponseMetadata': {
                'HTTPStatusCode': 200,
                'RequestId': 'request-id',
            },
        }
        outfile = self.files.full_path('cert.pem')
        cmdline = 'iot create-certificate-from-csr'
        cmdline += ' --certificate-signing-request "abc"'
        cmdline += ' --certificate-pem-outfile ' + outfile
        self.run_cmd(cmdline, 0)
        self.assertTrue(os.path.exists(outfile))
        with open(outfile) as fp:
            self.assertEqual('cert...', fp.read())

    def test_saves_files_for_create_keys_and_cert(self):
        self.parsed_response = {
            'certificatePem': 'cert...',
            'keyPair': {'PublicKey': 'public', 'PrivateKey': 'private'},
            'ResponseMetadata': {
                'HTTPStatusCode': 200,
                'RequestId': 'request-id',
            },
        }
        out_cert = self.files.full_path('cert.pem')
        out_pub = self.files.full_path('key_rsa.pub')
        out_priv = self.files.full_path('key_rsa')
        cmdline = 'iot create-keys-and-certificate'
        cmdline += ' --certificate-pem-outfile ' + out_cert
        cmdline += ' --public-key-outfile ' + out_pub
        cmdline += ' --private-key-outfile ' + out_priv
        self.run_cmd(cmdline, 0)
        self.assertTrue(os.path.exists(out_cert))
        self.assertTrue(os.path.exists(out_pub))
        self.assertTrue(os.path.exists(out_priv))
        with open(out_cert) as fp:
            self.assertEqual('cert...', fp.read())
        with open(out_pub) as fp:
            self.assertEqual('public', fp.read())
        with open(out_priv) as fp:
            self.assertEqual('private', fp.read())

    def test_bad_response(self):
        outfile = self.files.full_path('cert.pem')
        self.parsed_response = {
            'Error': {'Code': 'v1', 'Message': 'v2', 'Type': 'v3'},
            'ResponseMetadata': {
                'HTTPStatusCode': 403,
                'RequestId': 'request-id',
            },
        }
        self.http_response.status_code = 403
        cmdline = 'iot create-certificate-from-csr'
        cmdline += ' --certificate-signing-request "abc"'
        cmdline += ' --certificate-pem-outfile ' + outfile
        # The error message should be in the stderr.
        self.assert_params_for_cmd(
            cmdline,
            stderr_contains=self.parsed_response['Error']['Message'],
            expected_rc=254,
        )

    def test_ensures_file_is_writable_before_sending(self):
        outfile = os.sep.join(['', 'does', 'not', 'exist_', 'file.txt'])
        self.parsed_response = {}
        cmdline = 'iot create-certificate-from-csr'
        cmdline += ' --certificate-signing-request "abc"'
        cmdline += ' --certificate-pem-outfile ' + outfile
        self.assert_params_for_cmd(
            cmdline,
            stderr_contains='Unable to write to file: ',
            expected_rc=252,
        )



# Copyright 2013 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.


# Note that all of these functions can be found in the unit tests.
# The only difference is that these tests use botocore's actual session
# variables to communicate with s3 as these are integration tests.  Therefore,
# only tests that use sessions are included as integration tests.

import itertools
import os

import botocore.session
import pytest

from awscli.customizations.s3.filegenerator import FileGenerator, FileStat
from tests.integration.customizations.s3 import BaseS3IntegrationTest
from tests.unit.customizations.s3 import compare_files


@pytest.fixture(scope='module')
def s3_client(region):
    session = botocore.session.get_session()
    # Use the datetime and and blob parsing of the CLI
    factory = session.get_component('response_parser_factory')
    factory.set_parser_defaults(
        blob_parser=lambda x: x, timestamp_parser=lambda x: x
    )
    return session.create_client('s3', region_name=region)


@pytest.fixture()
def s3_files(shared_bucket, s3_utils):
    string1 = "This is a test."
    string2 = "This is another test."
    key1 = 'text1.txt'
    key2 = 'text2.txt'

    s3_utils.put_object(
        bucket_name=shared_bucket, key_name=key1, contents=string1
    )
    s3_utils.put_object(
        bucket_name=shared_bucket, key_name='another_directory/'
    )
    s3_utils.put_object(
        bucket_name=shared_bucket,
        key_name=f'another_directory/{key2}',
        contents=string2,
    )
    return [
        f'{shared_bucket}/{key1}',
        f'{shared_bucket}/another_directory/',
        f'{shared_bucket}/another_directory/{key2}',
    ]


class TestS3FileGenerator(BaseS3IntegrationTest):
    def test_s3_file(self, s3_client, s3_files):
        #
        # Generate a single s3 file
        # Note: Size and last update are not tested because s3 generates them.
        #
        input_s3_file = {
            'src': {'path': s3_files[0], 'type': 's3'},
            'dest': {'path': 'text1.txt', 'type': 'local'},
            'dir_op': False,
            'use_src_name': False,
        }
        expected_file_size = 15
        result_list = list(FileGenerator(s3_client, '').call(input_s3_file))
        file_stat = FileStat(
            src=s3_files[0],
            dest='text1.txt',
            compare_key='text1.txt',
            size=expected_file_size,
            last_update=result_list[0].last_update,
            src_type='s3',
            dest_type='local',
            operation_name='',
        )

        expected_list = [file_stat]
        assert len(result_list) == 1
        compare_files(result_list[0], expected_list[0])

    def test_s3_directory(self, s3_client, shared_bucket, s3_files):
        #
        # Generates s3 files under a common prefix. Also it ensures that
        # zero size files are ignored.
        # Note: Size and last update are not tested because s3 generates them.
        #
        input_s3_file = {
            'src': {'path': shared_bucket + '/', 'type': 's3'},
            'dest': {'path': '', 'type': 'local'},
            'dir_op': True,
            'use_src_name': True,
        }
        result_list = list(FileGenerator(s3_client, '').call(input_s3_file))
        file_stat = FileStat(
            src=s3_files[2],
            dest='another_directory' + os.sep + 'text2.txt',
            compare_key='another_directory/text2.txt',
            size=21,
            last_update=result_list[0].last_update,
            src_type='s3',
            dest_type='local',
            operation_name='',
        )
        file_stat2 = FileStat(
            src=s3_files[0],
            dest='text1.txt',
            compare_key='text1.txt',
            size=15,
            last_update=result_list[1].last_update,
            src_type='s3',
            dest_type='local',
            operation_name='',
        )

        expected_result = [file_stat, file_stat2]
        assert len(result_list) == 2
        compare_files(result_list[0], expected_result[0])
        compare_files(result_list[1], expected_result[1])

    def test_s3_delete_directory(self, s3_client, shared_bucket, s3_files):
        #
        # Generates s3 files under a common prefix. Also it ensures that
        # the directory itself is included because it is a delete command
        # Note: Size and last update are not tested because s3 generates them.
        #
        input_s3_file = {
            'src': {'path': shared_bucket + '/', 'type': 's3'},
            'dest': {'path': '', 'type': 'local'},
            'dir_op': True,
            'use_src_name': True,
        }
        result_list = list(
            FileGenerator(s3_client, 'delete').call(input_s3_file)
        )

        file_stat1 = FileStat(
            src=shared_bucket + '/another_directory/',
            dest='another_directory' + os.sep,
            compare_key='another_directory/',
            size=0,
            last_update=result_list[0].last_update,
            src_type='s3',
            dest_type='local',
            operation_name='delete',
        )
        file_stat2 = FileStat(
            src=s3_files[2],
            dest='another_directory' + os.sep + 'text2.txt',
            compare_key='another_directory/text2.txt',
            size=21,
            last_update=result_list[1].last_update,
            src_type='s3',
            dest_type='local',
            operation_name='delete',
        )
        file_stat3 = FileStat(
            src=s3_files[0],
            dest='text1.txt',
            compare_key='text1.txt',
            size=15,
            last_update=result_list[2].last_update,
            src_type='s3',
            dest_type='local',
            operation_name='delete',
        )

        expected_list = [file_stat1, file_stat2, file_stat3]
        assert len(result_list) == 3
        compare_files(result_list[0], expected_list[0])
        compare_files(result_list[1], expected_list[1])
        compare_files(result_list[2], expected_list[2])

    def test_page_size(self, s3_client, shared_bucket, s3_files):
        input_s3_file = {
            'src': {'path': shared_bucket + '/', 'type': 's3'},
            'dest': {'path': '', 'type': 'local'},
            'dir_op': True,
            'use_src_name': True,
        }
        file_gen = FileGenerator(s3_client, '', page_size=1).call(
            input_s3_file
        )
        limited_file_gen = itertools.islice(file_gen, 1)
        result_list = list(limited_file_gen)
        file_stat = FileStat(
            src=s3_files[2],
            dest='another_directory' + os.sep + 'text2.txt',
            compare_key='another_directory/text2.txt',
            size=21,
            last_update=result_list[0].last_update,
            src_type='s3',
            dest_type='local',
            operation_name='',
        )
        # Ensure only one item is returned from ``ListObjects``
        assert len(result_list) == 1
        compare_files(result_list[0], file_stat)



import pytest
from botocore.compat import json

import awscli.customizations.datapipeline.createdefaultroles as createdefaultroles
from awscli.customizations.datapipeline.constants import (
    DATAPIPELINE_DEFAULT_RESOURCE_ROLE_ASSUME_POLICY,
    DATAPIPELINE_DEFAULT_RESOURCE_ROLE_NAME,
    DATAPIPELINE_DEFAULT_SERVICE_ROLE_ASSUME_POLICY,
    DATAPIPELINE_DEFAULT_SERVICE_ROLE_NAME,
)
from awscli.customizations.datapipeline.translator import dict_to_string
from awscli.testutils import BaseAWSCommandParamsTest, mock, unittest


@pytest.mark.filterwarnings('ignore::UserWarning')
class TestCreateDefaultRole(BaseAWSCommandParamsTest):
    prefix = 'datapipeline create-default-roles'

    DATAPIPELINE_ROLE_POLICY = {
        "Statement": [
            {
                "Action": [
                    "cloudwatch:*",
                    "dynamodb:*",
                    "ec2:Describe*",
                    "elasticmapreduce:Describe*",
                    "rds:Describe*",
                    "s3:*",
                    "sdb:*",
                    "sns:*",
                    "sqs:*",
                ],
                "Effect": "Allow",
                "Resource": ["*"],
            }
        ]
    }

    CREATE_DATAPIPELINE_ROLE_RESULT = {
        "Role": {
            "AssumeRolePolicyDocument": {
                "Version": "2008-10-17",
                "Statement": [
                    {
                        "Action": "sts:AssumeRole",
                        "Sid": "",
                        "Effect": "Allow",
                        "Principal": {"Service": "ec2.amazonaws.com"},
                    }
                ],
            },
            "RoleId": "AROAJG7O4RNNSRINMF6DI",
            "CreateDate": "2014-05-01T23:47:14.552Z",
            "RoleName": DATAPIPELINE_DEFAULT_SERVICE_ROLE_NAME,
            "Path": "/",
            "Arn": "arn:aws:iam::176430881729:role/"
            + DATAPIPELINE_DEFAULT_SERVICE_ROLE_NAME,
        }
    }

    CONSTRUCTED_RESULT_OUTPUT = [
        {
            "Role": CREATE_DATAPIPELINE_ROLE_RESULT['Role'],
            "RolePolicy": DATAPIPELINE_ROLE_POLICY,
        }
    ]

    # Use case: Default roles exists
    # Expected results: No Operation performed for creation, except calls made
    # for verifying existence of roles
    def test_default_roles_exist(self):
        cmdline = self.prefix

        self.run_cmd(cmdline, expected_rc=0)
        self.assertEqual(len(self.operations_called), 3)

        self.assertEqual(self.operations_called[0][0].name, 'GetRole')
        self.assertEqual(
            self.operations_called[0][1]['RoleName'],
            DATAPIPELINE_DEFAULT_SERVICE_ROLE_NAME,
        )

    # Use case: Default roles do not exist
    # Expected results: Operations are performed by the client to verify
    # existence of roles and then creation of roles (Service role,
    # resource role and instance profile)
    @mock.patch(
        'awscli.customizations.datapipeline.createdefaultroles.'
        'CreateDefaultRoles._construct_result'
    )
    @mock.patch(
        'awscli.customizations.datapipeline.createdefaultroles.'
        'CreateDefaultRoles._check_if_role_exists'
    )
    @mock.patch(
        'awscli.customizations.datapipeline.createdefaultroles.'
        'CreateDefaultRoles._check_if_instance_profile_exists'
    )
    @mock.patch(
        'awscli.customizations.datapipeline.createdefaultroles.'
        'CreateDefaultRoles._get_role_policy'
    )
    def test_default_roles_not_exist(
        self,
        get_rp_patch,
        role_exists_patch,
        instance_profile_exists_patch,
        construct_result_patch,
    ):
        get_rp_patch.return_value = False
        instance_profile_exists_patch.return_value = False
        role_exists_patch.return_value = False
        construct_result_patch.return_value = []

        self.run_cmd(self.prefix, expected_rc=0)
        self.assertEqual(len(self.operations_called), 6)

        self.assertEqual(self.operations_called[0][0].name, 'CreateRole')
        self.assertEqual(
            self.operations_called[0][1]['RoleName'],
            DATAPIPELINE_DEFAULT_SERVICE_ROLE_NAME,
        )
        self.assertEqual(
            self.operations_called[0][1]['AssumeRolePolicyDocument'],
            dict_to_string(DATAPIPELINE_DEFAULT_SERVICE_ROLE_ASSUME_POLICY),
        )

        self.assertEqual(self.operations_called[1][0].name, 'AttachRolePolicy')
        self.assertEqual(
            self.operations_called[1][1]['PolicyArn'],
            (createdefaultroles.DATAPIPELINE_DEFAULT_SERVICE_ROLE_ARN),
        )
        self.assertEqual(
            self.operations_called[1][1]['RoleName'],
            DATAPIPELINE_DEFAULT_SERVICE_ROLE_NAME,
        )

        self.assertEqual(self.operations_called[2][0].name, 'CreateRole')
        self.assertEqual(
            self.operations_called[2][1]['RoleName'],
            DATAPIPELINE_DEFAULT_RESOURCE_ROLE_NAME,
        )
        self.assertEqual(
            self.operations_called[2][1]['AssumeRolePolicyDocument'],
            dict_to_string(DATAPIPELINE_DEFAULT_RESOURCE_ROLE_ASSUME_POLICY),
        )

        self.assertEqual(self.operations_called[3][0].name, 'AttachRolePolicy')
        self.assertEqual(
            self.operations_called[3][1]['PolicyArn'],
            (createdefaultroles.DATAPIPELINE_DEFAULT_RESOURCE_ROLE_ARN),
        )
        self.assertEqual(
            self.operations_called[3][1]['RoleName'],
            DATAPIPELINE_DEFAULT_RESOURCE_ROLE_NAME,
        )

        self.assertEqual(
            self.operations_called[4][0].name, 'CreateInstanceProfile'
        )
        self.assertEqual(
            self.operations_called[4][1]['InstanceProfileName'],
            DATAPIPELINE_DEFAULT_RESOURCE_ROLE_NAME,
        )

        self.assertEqual(
            self.operations_called[5][0].name, 'AddRoleToInstanceProfile'
        )
        self.assertEqual(
            self.operations_called[5][1]['InstanceProfileName'],
            DATAPIPELINE_DEFAULT_RESOURCE_ROLE_NAME,
        )
        self.assertEqual(
            self.operations_called[5][1]['RoleName'],
            DATAPIPELINE_DEFAULT_RESOURCE_ROLE_NAME,
        )

    # Use case: Creating only DataPipeline service role
    # Expected output: The service role is created displaying a message
    # to the customer that a particular role with a policy has been created
    @mock.patch(
        'awscli.customizations.datapipeline.createdefaultroles.'
        'CreateDefaultRoles._get_role_policy'
    )
    @mock.patch(
        'awscli.customizations.datapipeline.createdefaultroles.'
        'CreateDefaultRoles._create_role_with_role_policy'
    )
    @mock.patch(
        'awscli.customizations.datapipeline.createdefaultroles.'
        'CreateDefaultRoles._check_if_instance_profile_exists'
    )
    @mock.patch(
        'awscli.customizations.datapipeline.createdefaultroles.'
        'CreateDefaultRoles._check_if_role_exists'
    )
    def test_constructed_result(
        self,
        role_exists_patch,
        instance_profile_exists_patch,
        create_role_patch,
        get_role_policy_patch,
    ):
        role_exists_patch.side_effect = self.toggle_for_check_if_exists
        instance_profile_exists_patch.return_value = True
        create_role_patch.return_value = self.CREATE_DATAPIPELINE_ROLE_RESULT
        get_role_policy_patch.return_value = self.DATAPIPELINE_ROLE_POLICY

        result = self.run_cmd(self.prefix, 0)
        expected_output = (
            json.dumps(self.CONSTRUCTED_RESULT_OUTPUT, indent=4) + '\n'
        )
        self.assertEqual(result[0], expected_output)

    def toggle_for_check_if_exists(self, *args):
        if args[0] == DATAPIPELINE_DEFAULT_RESOURCE_ROLE_NAME:
            return False
        else:
            return True


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# Copyright 2012-2013 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from awscli.testutils import BaseAWSCommandParamsTest


class TestCreatePlatformApplication(BaseAWSCommandParamsTest):
    prefix = 'sns create-platform-application'

    def test_gcm_shorthand(self):
        cmdline = self.prefix
        cmdline += ' --name gcmpushapp'
        cmdline += ' --platform GCM'
        cmdline += ' --attributes '
        cmdline += 'PlatformCredential=foo,'
        cmdline += 'PlatformPrincipal=bar'
        result = {
            'Name': 'gcmpushapp',
            'Platform': 'GCM',
            'Attributes': {
                'PlatformCredential': 'foo',
                'PlatformPrincipal': 'bar',
            },
        }
        self.assert_params_for_cmd(cmdline, result)

    def test_gcm_json(self):
        cmdline = self.prefix
        cmdline += ' --name gcmpushapp'
        cmdline += ' --platform GCM'
        cmdline += ' --attributes '
        cmdline += (
            '{"PlatformCredential":"AIzaSyClE2lcV2zEKTLYYo645zfk2jhQPFeyxDo",'
            '"PlatformPrincipal":"There+is+no+principal+for+GCM"}'
        )
        result = {
            'Name': 'gcmpushapp',
            'Platform': 'GCM',
            'Attributes': {
                'PlatformCredential': 'AIzaSyClE2lcV2zEKTLYYo645zfk2jhQPFeyxDo',
                'PlatformPrincipal': 'There+is+no+principal+for+GCM',
            },
        }
        self.assert_params_for_cmd(cmdline, result)


if __name__ == "__main__":
    unittest.main()



# Copyright 2017 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import uuid

from botocore.history import HistoryRecorder

from awscli.clidriver import AWSCLIEntryPoint
from awscli.testutils import (
    BaseAWSCommandParamsTest,
    FileCreator,
    create_clidriver,
    mock,
)


class BaseHistoryCommandParamsTest(BaseAWSCommandParamsTest):
    def setUp(self):
        history_recorder = self._make_clean_history_recorder()
        super(BaseHistoryCommandParamsTest, self).setUp()
        self.history_recorder = history_recorder
        self.files = FileCreator()
        config_contents = '[default]\n' 'cli_history = enabled'
        self.environ['AWS_CONFIG_FILE'] = self.files.create_file(
            'config', config_contents
        )
        self.environ['AWS_CLI_HISTORY_FILE'] = self.files.create_file(
            'history.db', ''
        )
        self.driver = create_clidriver()
        self.entry_point = AWSCLIEntryPoint(self.driver)

    def _make_clean_history_recorder(self):
        # This is to ensure that for each new test run the CLI is using
        # a brand new HistoryRecorder as this is global so previous test
        # runs could have injected handlers onto it as all of the tests
        # are ran in the same process.
        history_recorder = HistoryRecorder()

        # The HISTORY_RECORDER is instantiated on module import before we
        # doing any patching which means we cannot simply patch
        # botocore.get_global_history_recorder as the objects are already
        # instantiated as so we have to individually patch each one of these...
        self._apply_history_recorder_patch(
            'awscli.clidriver', history_recorder
        )
        self._apply_history_recorder_patch(
            'awscli.customizations.history', history_recorder
        )
        return history_recorder

    def _apply_history_recorder_patch(self, module, history_recorder):
        patch_history_recorder = mock.patch(
            module + '.HISTORY_RECORDER', history_recorder
        )
        patch_history_recorder.start()
        self.addCleanup(patch_history_recorder.stop)

    def _cleanup_db_connections(self):
        # Reaching into private data to close out the database connection.
        # Windows won't let us delete the tempdir until these connections are
        # closed in the tearDown step and we have no other way of forcing
        # them to close.
        handlers = self.history_recorder._handlers
        for handler in handlers:
            handler._writer.close()

    def tearDown(self):
        super(BaseHistoryCommandParamsTest, self).tearDown()
        self._cleanup_db_connections()
        self.files.remove_all()



import binascii
import copy
import hashlib
import json
import os
from io import BytesIO

from awscrt.crypto import RSASignatureAlgorithm

from awscli.testutils import BaseAWSCommandParamsTest
from tests import PublicPrivateKeyLoader

from . import get_private_key_path, get_public_key_path

SAMPLE_PUBLIC_KEY_FINGERPRINT = "67b9fa73676d86966b449dd677850753"
SAMPLE_PUBLIC_KEY = (
    "MIIBCgKCAQEA1OxVzcylHnxhysU41BcHucQn47bywceJEPkze53R"
    "ZUaYtvTAXYf0ONbdk/yQ8c5Bq6l5nL0qjT1UNhVfccCsTaToL58w"
    "PePg9U29D+MtiQXz6yecW7Rzs5rP5i4EU2YDPoYaphIVowYZjkHy"
    "C72L/2lQ/Q5rSotIjUdCq6Wv6i7bFHLiCNtgP95xfL9pxytAyGdC"
    "zHDBJb/H60x7q/wqO6/HaZI38aY7piHqazd2vpaNxPShHyvpdR0P"
    "29tNFxo+5V4VVW2/QC1/QLTLLAzQNiO+ZOpsHa8rgYmPv0YI4sFA"
    "9vFhHyUdh9skopO7Jg/gAjAdtzuLCnXpfuwQ/+EXnQIDAQAB"
)
SAMPLE_EXPORT_FILE = "# Execution Time: 2022-05-24T21:51:51.723951Z "
SAMPLE_HASH_VALUE = (
    "3b9c358f36f0a31b6ad3e14f309c7cf198ac9246e8316f9ce543" "d5b19ac02b80"
)
SAMPLE_SIGNING_FILE = {
    "region": "us-east-1",
    "files": [
        {"fileHashValue": SAMPLE_HASH_VALUE, "fileName": "result_1.csv.gz"}
    ],
    "hashAlgorithm": "SHA-256",
    "publicKeyFingerprint": "fingerprint",
    "signatureAlgorithm": "SHA256withRSA",
    "hashSignature": "signature",
    "queryCompleteTime": "2022-05-10T22:06:30Z",
}

SAMPLE_S3_BUCKET_NAME = "lake-bucket-name"
SAMPLE_S3_EXPORT_FILE_PREFIX = "lake-export-prefix/"


class TestVerifyQueryResults(BaseAWSCommandParamsTest):
    def test_get_file_form_s3_happy_case(self):
        (
            public_key,
            private_key,
        ) = PublicPrivateKeyLoader.load_private_key_and_public_key(
            get_private_key_path(), get_public_key_path()
        )

        sign_file = copy.deepcopy(SAMPLE_SIGNING_FILE)
        signature = private_key.sign(
            signature_algorithm=RSASignatureAlgorithm.PKCS1_5_SHA256,
            digest=hashlib.sha256(SAMPLE_HASH_VALUE.encode()).digest(),
        )
        sign_file["hashSignature"] = binascii.hexlify(signature).decode()

        self.parsed_responses = [
            {"Body": BytesIO(json.dumps(sign_file).encode("utf-8"))},
            {"Body": BytesIO(b"file")},
            {
                "PublicKeyList": [
                    {"Fingerprint": "fingerprint", "Value": public_key}
                ]
            },
        ]

        stdout, stderr, rc = self.run_cmd(
            f"cloudtrail verify-query-results --s3-bucket={SAMPLE_S3_BUCKET_NAME}"
            f" --s3-prefix={SAMPLE_S3_EXPORT_FILE_PREFIX} "
        )

        self.assertIn(
            "Successfully validated sign and query result files\n", stdout
        )
        self.assertEqual(self.operations_called[0][0].name, "GetObject")
        self.assertEqual(
            self.operations_called[0][1]["Bucket"], SAMPLE_S3_BUCKET_NAME
        )
        self.assertEqual(
            self.operations_called[0][1]["Key"],
            "lake-export-prefix/result_sign.json",
        )

        self.assertEqual(self.operations_called[1][0].name, "GetObject")
        self.assertEqual(
            self.operations_called[1][1]["Bucket"], SAMPLE_S3_BUCKET_NAME
        )
        self.assertEqual(
            self.operations_called[1][1]["Key"],
            "lake-export-prefix/result_1.csv.gz",
        )

        self.assertEqual(self.operations_called[2][0].name, "ListPublicKeys")

    def test_get_file_form_s3_invalid_signature(self):
        sign_file = copy.deepcopy(SAMPLE_SIGNING_FILE)
        (
            public_key,
            private_key,
        ) = PublicPrivateKeyLoader.load_private_key_and_public_key(
            get_private_key_path(), get_public_key_path()
        )
        signature = private_key.sign(
            signature_algorithm=RSASignatureAlgorithm.PKCS1_5_SHA256,
            digest=hashlib.sha256(b"123").digest(),
        )

        sign_file["hashSignature"] = binascii.hexlify(signature).decode()

        self.parsed_responses = [
            {"Body": BytesIO(json.dumps(sign_file).encode("utf-8"))},
            {"Body": BytesIO(b"file")},
            {
                "PublicKeyList": [
                    {"Fingerprint": "fingerprint", "Value": public_key}
                ]
            },
        ]

        stdout, stderr, rc = self.run_cmd(
            f"cloudtrail verify-query-results --s3-bucket={SAMPLE_S3_BUCKET_NAME}"
            f" --s3-prefix={SAMPLE_S3_EXPORT_FILE_PREFIX} ",
            255,
        )
        self.assertIn("Invalid signature in sign file", stderr)

    def test_get_file_form_s3_invalid_sign_file(self):
        (
            public_key,
            _,
        ) = PublicPrivateKeyLoader.load_private_key_and_public_key(
            get_private_key_path(), get_public_key_path()
        )
        self.parsed_responses = [
            {"Body": BytesIO(b"123")},
            {"Body": BytesIO(b"file")},
            {
                "PublicKeyList": [
                    {"Fingerprint": "fingerprint", "Value": public_key}
                ]
            },
        ]

        stdout, stderr, rc = self.run_cmd(
            f"cloudtrail verify-query-results --s3-bucket={SAMPLE_S3_BUCKET_NAME}"
            f" --s3-prefix={SAMPLE_S3_EXPORT_FILE_PREFIX} "
            f" --endpoint-url=https://testurl/ ",
            255,
        )
        self.assertIn("Invalid sign file provided", stderr)

        self.assertEqual(self.operations_called[0][0].name, "GetObject")
        self.assertEqual(len(self.operations_called), 1)

        self.assertNotIn("testurl", self.last_request_dict["url"])

    def test_invalid_parameter_both_empty(self):
        stdout, stderr, rc = self.run_cmd(
            "cloudtrail verify-query-results ", 252
        )
        self.assertIn(
            "Require parameter --s3-bucket or --local-export-path.", stderr
        )
        self.assertEqual(len(self.operations_called), 0)

    def test_invalid_parameter_both_provided(self):
        stdout, stderr, rc = self.run_cmd(
            "cloudtrail verify-query-results "
            "--s3-bucket=test-bucket "
            "--local-export-path=/test/",
            252,
        )
        self.assertIn(
            "Parameter --local-export-path can not be specified with parameter"
            " --s3-bucket nor --s3-prefix.",
            stderr,
        )
        self.assertEqual(len(self.operations_called), 0)

    def test_get_file_form_s3_incorrect_hash_value(self):
        sign_file = copy.deepcopy(SAMPLE_SIGNING_FILE)
        (
            public_key,
            private_key,
        ) = PublicPrivateKeyLoader.load_private_key_and_public_key(
            get_private_key_path(), get_public_key_path()
        )
        signature = private_key.sign(
            signature_algorithm=RSASignatureAlgorithm.PKCS1_5_SHA256,
            digest=hashlib.sha256(SAMPLE_HASH_VALUE.encode()).digest(),
        )
        sign_file["hashSignature"] = binascii.hexlify(signature).decode()

        self.parsed_responses = [
            {"Body": BytesIO(json.dumps(sign_file).encode("utf-8"))},
            {"Body": BytesIO(b"123")},
            {
                "PublicKeyList": [
                    {"Fingerprint": "fingerprint", "Value": public_key}
                ]
            },
        ]

        stdout, stderr, rc = self.run_cmd(
            f"cloudtrail verify-query-results --s3-bucket={SAMPLE_S3_BUCKET_NAME}"
            f" --s3-prefix={SAMPLE_S3_EXPORT_FILE_PREFIX} ",
            255,
        )
        self.assertIn(
            "has inconsistent hash value with hash value recorded", stderr
        )

        self.assertEqual(self.operations_called[0][0].name, "GetObject")
        self.assertEqual(self.operations_called[1][0].name, "GetObject")
        self.assertEqual(len(self.operations_called), 2)

    def test_get_file_form_local_form_local_success(self):
        self.parsed_responses = [
            {
                "PublicKeyList": [
                    {
                        "Fingerprint": SAMPLE_PUBLIC_KEY_FINGERPRINT,
                        "Value": SAMPLE_PUBLIC_KEY,
                    }
                ]
            }
        ]

        current_dir = os.path.dirname(os.path.realpath(__file__))
        local_export_file_path = os.path.join(current_dir, "test_resource")

        stdout, stderr, rc = self.run_cmd(
            f"cloudtrail verify-query-results --local-export-path={local_export_file_path}"
        )
        self.assertIn(
            "Successfully validated sign and query result files\n", stdout
        )

        self.assertEqual(self.operations_called[0][0].name, "ListPublicKeys")
        self.assertEqual(len(self.operations_called), 1)

    def test_override_endpoint_url(self):
        self.parsed_responses = [
            {
                "PublicKeyList": [
                    {
                        "Fingerprint": SAMPLE_PUBLIC_KEY_FINGERPRINT,
                        "Value": SAMPLE_PUBLIC_KEY,
                    }
                ]
            }
        ]

        current_dir = os.path.dirname(os.path.realpath(__file__))
        local_export_file_path = os.path.join(current_dir, "test_resource")

        stdout, stderr, rc = self.run_cmd(
            f"cloudtrail verify-query-results --local-export-path={local_export_file_path}"
            f" --endpoint-url=https://testurl/ "
        )

        self.assertIn(
            "Successfully validated sign and query result files\n", stdout
        )
        self.assertEqual(self.last_request_dict["url"], "https://testurl/")



import copy
import os
import platform
import re
import subprocess
import time
from configparser import RawConfigParser
from io import StringIO
from urllib.parse import urlsplit

from botocore.utils import parse_timestamp

from awscli.compat import urlparse
from awscli.customizations.codeartifact.login import CodeArtifactLogin
from awscli.testutils import FileCreator, mock, unittest
from tests import AWSRequest, AWSResponse, CLIRunner


class TestCodeArtifactLogin(unittest.TestCase):
    prefix = ['codeartifact', 'login']

    def setUp(self):
        self.file_creator = FileCreator()
        self.test_pypi_rc_path = self.file_creator.full_path('pypirc')
        if not os.path.isdir(os.path.dirname(self.test_pypi_rc_path)):
            os.makedirs(os.path.dirname(self.test_pypi_rc_path))

        self.domain = 'domain'
        self.domain_owner = 'domain-owner'
        self.repository = 'repository'
        self.endpoint_type = 'ipv4'
        self.auth_token = 'auth-token'
        self.namespace = 'namespace'
        self.nuget_index_url_fmt = '{endpoint}v3/index.json'
        self.nuget_source_name = self.domain + '/' + self.repository
        self.duration = 3600
        self.expiration = time.time() + self.duration
        self.expiration_as_datetime = parse_timestamp(self.expiration)

        self.pypi_rc_path_patch = mock.patch(
            'awscli.customizations.codeartifact.login.TwineLogin'
            '.get_pypi_rc_path'
        )
        self.pypi_rc_path_mock = self.pypi_rc_path_patch.start()
        self.pypi_rc_path_mock.return_value = self.test_pypi_rc_path

        self.test_netrc_path = self.file_creator.full_path('.netrc')
        self.get_netrc_path_patch = mock.patch(
            'awscli.customizations.codeartifact.login.SwiftLogin'
            '.get_netrc_path'
        )
        self.get_netrc_mock = self.get_netrc_path_patch.start()
        self.get_netrc_mock.return_value = self.test_netrc_path

        self.subprocess_patch = mock.patch('subprocess.run')
        self.subprocess_mock = self.subprocess_patch.start()
        self.subprocess_check_output_patch = mock.patch(
            'subprocess.check_output'
        )
        self.subprocess_check_out_mock = (
            self.subprocess_check_output_patch.start()
        )
        self.cli_runner = CLIRunner()

    def tearDown(self):
        self.pypi_rc_path_patch.stop()
        self.subprocess_check_output_patch.stop()
        self.get_netrc_path_patch.stop()
        self.subprocess_patch.stop()
        self.file_creator.remove_all()

    def _setup_cmd(
        self,
        tool,
        include_domain_owner=False,
        dry_run=False,
        include_endpoint_type=False,
        include_duration_seconds=False,
        include_namespace=False,
    ):
        package_format = CodeArtifactLogin.TOOL_MAP[tool]['package_format']
        self.endpoint = (
            f'https://{self.domain}-{self.domain_owner}.codeartifact.aws.'
            f'a2z.com/{package_format}/{self.repository}/'
        )

        cmdline = copy.copy(self.prefix)
        cmdline.extend(
            [
                '--domain',
                self.domain,
                '--repository',
                self.repository,
                '--tool',
                tool,
            ]
        )

        if include_domain_owner:
            cmdline.extend(['--domain-owner', self.domain_owner])

        if include_endpoint_type:
            cmdline.extend(['--endpoint-type', self.endpoint_type])

        if dry_run:
            cmdline.append('--dry-run')

        if include_duration_seconds:
            cmdline.extend(['--duration-seconds', str(self.duration)])

        if include_namespace:
            cmdline.extend(['--namespace', self.namespace])

        self.cli_runner.add_response(
            AWSResponse(
                service_name='codeartifact',
                operation_name='GetAuthorizationToken',
                parsed_response={
                    "authorizationToken": self.auth_token,
                    "expiration": self.expiration_as_datetime,
                },
            )
        )
        self.cli_runner.add_response(
            AWSResponse(
                service_name='codeartifact',
                operation_name='GetRepositoryEndpoint',
                parsed_response={"repositoryEndpoint": self.endpoint},
            )
        )

        return cmdline

    def _get_swift_commands(self, scope=None, token=None):
        commands = []
        set_registry_command = [
            'swift',
            'package-registry',
            'set',
            self.endpoint,
        ]
        if scope is not None:
            set_registry_command.extend(['--scope', scope])
        commands.append(set_registry_command)

        login_registry_command = [
            'swift',
            'package-registry',
            'login',
            f'{self.endpoint}login',
        ]
        if token is not None:
            login_registry_command.extend(['--token', token])
        commands.append(login_registry_command)

        return commands

    def _get_nuget_commands(self):
        nuget_index_url = self.nuget_index_url_fmt.format(
            endpoint=self.endpoint
        )

        commands = []
        commands.append(
            [
                'nuget',
                'sources',
                'add',
                '-name',
                self.nuget_source_name,
                '-source',
                nuget_index_url,
                '-username',
                'aws',
                '-password',
                self.auth_token,
            ]
        )
        return commands

    def _get_dotnet_commands(self):
        nuget_index_url = self.nuget_index_url_fmt.format(
            endpoint=self.endpoint
        )

        commands = []
        commands.append(
            [
                'dotnet',
                'nuget',
                'add',
                'source',
                nuget_index_url,
                '--name',
                self.nuget_source_name,
                '--username',
                'aws',
                '--password',
                self.auth_token,
            ]
        )
        return commands

    def _get_npm_commands(self, **kwargs):
        npm_cmd = (
            'npm.cmd' if platform.system().lower() == 'windows' else 'npm'
        )

        repo_uri = urlsplit(self.endpoint)
        always_auth_config = f'//{repo_uri.netloc}{repo_uri.path}:always-auth'
        auth_token_config = f'//{repo_uri.netloc}{repo_uri.path}:_authToken'

        scope = kwargs.get('scope')
        registry = f'{scope}:registry' if scope else 'registry'

        commands = []
        commands.append([npm_cmd, 'config', 'set', registry, self.endpoint])
        commands.append([npm_cmd, 'config', 'set', always_auth_config, 'true'])
        commands.append(
            [npm_cmd, 'config', 'set', auth_token_config, self.auth_token]
        )

        return commands

    def _get_pip_commands(self):
        pip_index_url_fmt = '{scheme}://aws:{auth_token}@{netloc}{path}simple/'
        repo_uri = urlsplit(self.endpoint)
        pip_index_url = pip_index_url_fmt.format(
            scheme=repo_uri.scheme,
            auth_token=self.auth_token,
            netloc=repo_uri.netloc,
            path=repo_uri.path,
        )

        return [['pip', 'config', 'set', 'global.index-url', pip_index_url]]

    def _get_twine_commands(self):
        default_pypi_rc_fmt = '''\
[distutils]
index-servers=
    pypi
    codeartifact

[codeartifact]
repository: {repository_endpoint}
username: aws
password: {auth_token}'''
        default_pypi_rc = default_pypi_rc_fmt.format(
            repository_endpoint=self.endpoint, auth_token=self.auth_token
        )

        pypi_rc = RawConfigParser()
        if os.path.exists(self.test_pypi_rc_path):
            pypi_rc.read(self.test_pypi_rc_path)
            index_servers = pypi_rc.get('distutils', 'index-servers')
            servers = [
                server.strip()
                for server in index_servers.split('\n')
                if server.strip() != ''
            ]

            if 'codeartifact' not in servers:
                servers.append('codeartifact')
                pypi_rc.set(
                    'distutils', 'index-servers', '\n' + '\n'.join(servers)
                )

            if 'codeartifact' not in pypi_rc.sections():
                pypi_rc.add_section('codeartifact')

            pypi_rc.set('codeartifact', 'repository', self.endpoint)
            pypi_rc.set('codeartifact', 'username', 'aws')
            pypi_rc.set('codeartifact', 'password', self.auth_token)
        else:
            pypi_rc.read_string(default_pypi_rc)

        pypi_rc_stream = StringIO()
        pypi_rc.write(pypi_rc_stream)
        pypi_rc_str = pypi_rc_stream.getvalue()
        pypi_rc_stream.close()

        return pypi_rc_str

    def _assert_expiration_printed_to_stdout(self, stdout):
        self.assertEqual(
            self.expiration_as_datetime.strftime("%Y-%m-%d %H:%M:%S"),
            stdout.split("at ")[1][0:19],
        )

    def _assert_operations_called(
        self,
        package_format,
        result,
        include_domain_owner=False,
        include_duration_seconds=False,
        include_endpoint_type=False,
    ):
        get_auth_token_kwargs = {'domain': self.domain}
        get_repo_endpoint_kwargs = {
            'domain': self.domain,
            'repository': self.repository,
            'format': package_format,
        }

        if include_domain_owner:
            get_auth_token_kwargs['domainOwner'] = self.domain_owner
            get_repo_endpoint_kwargs['domainOwner'] = self.domain_owner

        if include_endpoint_type:
            get_repo_endpoint_kwargs['endpointType'] = self.endpoint_type

        if include_duration_seconds:
            get_auth_token_kwargs['durationSeconds'] = self.duration

        self.assertEqual(
            result.aws_requests,
            [
                AWSRequest(
                    service_name='codeartifact',
                    operation_name='GetAuthorizationToken',
                    params=get_auth_token_kwargs,
                ),
                AWSRequest(
                    service_name='codeartifact',
                    operation_name='GetRepositoryEndpoint',
                    params=get_repo_endpoint_kwargs,
                ),
            ],
        )

    def _assert_subprocess_execution(self, commands):
        expected_calls = [
            mock.call(command, capture_output=True, check=True)
            for command in commands
        ]
        self.subprocess_mock.assert_has_calls(expected_calls, any_order=True)

    def _assert_subprocess_check_output_execution(self, commands):
        expected_calls = [
            mock.call(
                command,
                stderr=subprocess.PIPE,
            )
            for command in commands
        ]
        self.subprocess_check_out_mock.assert_has_calls(
            expected_calls, any_order=True
        )

    def _assert_dry_run_execution(self, commands, stdout):
        self.subprocess_mock.assert_not_called()
        for command in commands:
            self.assertIn(' '.join(command), stdout)

    def _assert_pypi_rc_has_expected_content(
        self, pypi_rc_str, server, repo_url=None, username=None, password=None
    ):
        pypi_rc = RawConfigParser()
        pypi_rc.read_string(pypi_rc_str)

        self.assertIn('distutils', pypi_rc.sections())
        self.assertIn('index-servers', pypi_rc.options('distutils'))
        index_servers = pypi_rc.get('distutils', 'index-servers')
        index_servers = [
            index_server.strip()
            for index_server in index_servers.split('\n')
            if index_server.strip() != ''
        ]
        self.assertIn(server, index_servers)

        if repo_url or username or password:
            self.assertIn(server, pypi_rc.sections())

        if repo_url:
            self.assertIn('repository', pypi_rc.options(server))
            self.assertEqual(pypi_rc.get(server, 'repository'), repo_url)

        if username:
            self.assertIn('username', pypi_rc.options(server))
            self.assertEqual(pypi_rc.get(server, 'username'), username)

        if password:
            self.assertIn('password', pypi_rc.options(server))
            self.assertEqual(pypi_rc.get(server, 'password'), password)

    def _assert_netrc_has_expected_content(self):
        with open(self.test_netrc_path) as f:
            actual_contents = f.read()

        hostname = urlparse.urlparse(self.endpoint).hostname
        expected_contents = (
            f'machine {hostname} login token password {self.auth_token}\n'
        )
        self.assertEqual(expected_contents, actual_contents)

    @mock.patch('awscli.customizations.codeartifact.login.is_macos', True)
    def test_swift_login_without_domain_owner_macos(self):
        cmdline = self._setup_cmd(tool='swift')
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(package_format='swift', result=result)
        self._assert_expiration_printed_to_stdout(result.stdout)
        self._assert_subprocess_execution(
            self._get_swift_commands(token=self.auth_token)
        )
        self.assertFalse(os.path.exists(self.test_netrc_path))

    @mock.patch('awscli.customizations.codeartifact.login.is_macos', False)
    def test_swift_login_without_domain_owner_non_macos(self):
        cmdline = self._setup_cmd(tool='swift')
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(package_format='swift', result=result)
        self._assert_expiration_printed_to_stdout(result.stdout)
        self._assert_subprocess_execution(self._get_swift_commands())
        self._assert_netrc_has_expected_content()

    @mock.patch('awscli.customizations.codeartifact.login.is_macos', False)
    def test_swift_login_without_domain_owner_dry_run(self):
        cmdline = self._setup_cmd(tool='swift', dry_run=True)
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(package_format='swift', result=result)
        self._assert_dry_run_execution(
            self._get_swift_commands(), result.stdout
        )
        self.assertFalse(os.path.exists(self.test_netrc_path))

    @mock.patch('awscli.customizations.codeartifact.login.is_macos', True)
    def test_swift_login_with_domain_owner_macos(self):
        cmdline = self._setup_cmd(tool='swift', include_domain_owner=True)
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(
            package_format='swift',
            result=result,
            include_domain_owner=True,
            include_duration_seconds=False,
        )
        self._assert_expiration_printed_to_stdout(result.stdout)
        self._assert_subprocess_execution(
            self._get_swift_commands(token=self.auth_token)
        )

    @mock.patch('awscli.customizations.codeartifact.login.is_macos', False)
    def test_swift_login_with_domain_owner_non_macos(self):
        cmdline = self._setup_cmd(tool='swift', include_domain_owner=True)
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(
            package_format='swift',
            result=result,
            include_domain_owner=True,
            include_duration_seconds=False,
        )
        self._assert_expiration_printed_to_stdout(result.stdout)
        self._assert_subprocess_execution(self._get_swift_commands())
        self._assert_netrc_has_expected_content()

    @mock.patch('awscli.customizations.codeartifact.login.is_macos', True)
    def test_swift_login_with_domain_owner_duration_macos(self):
        cmdline = self._setup_cmd(
            tool='swift',
            include_domain_owner=True,
            include_duration_seconds=True,
        )
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(
            package_format='swift',
            result=result,
            include_domain_owner=True,
            include_duration_seconds=True,
        )
        self._assert_expiration_printed_to_stdout(result.stdout)
        self._assert_subprocess_execution(
            self._get_swift_commands(token=self.auth_token)
        )

    @mock.patch('awscli.customizations.codeartifact.login.is_macos', False)
    def test_swift_login_with_domain_owner_duration_non_macos(self):
        cmdline = self._setup_cmd(
            tool='swift',
            include_domain_owner=True,
            include_duration_seconds=True,
        )
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(
            package_format='swift',
            result=result,
            include_domain_owner=True,
            include_duration_seconds=True,
        )
        self._assert_expiration_printed_to_stdout(result.stdout)
        self._assert_subprocess_execution(self._get_swift_commands())
        self._assert_netrc_has_expected_content()

    @mock.patch('awscli.customizations.codeartifact.login.is_macos', False)
    def test_swift_login_with_domain_owner_dry_run(self):
        cmdline = self._setup_cmd(
            tool='swift', include_domain_owner=True, dry_run=True
        )
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(
            package_format='swift', result=result, include_domain_owner=True
        )
        self._assert_dry_run_execution(
            self._get_swift_commands(), result.stdout
        )
        self.assertFalse(os.path.exists(self.test_netrc_path))

    @mock.patch('awscli.customizations.codeartifact.login.is_macos', True)
    def test_swift_login_with_domain_owner_duration_dry_run(self):
        cmdline = self._setup_cmd(
            tool='swift',
            include_domain_owner=True,
            include_duration_seconds=True,
            dry_run=True,
        )
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(
            package_format='swift',
            result=result,
            include_domain_owner=True,
            include_duration_seconds=True,
        )
        self._assert_dry_run_execution(
            self._get_swift_commands(token=self.auth_token), result.stdout
        )

    @mock.patch('awscli.customizations.codeartifact.login.is_macos', True)
    def test_swift_login_with_namespace_macos(self):
        cmdline = self._setup_cmd(tool='swift', include_namespace=True)
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(package_format='swift', result=result)
        self._assert_expiration_printed_to_stdout(result.stdout)
        self._assert_subprocess_execution(
            self._get_swift_commands(
                scope=self.namespace, token=self.auth_token
            )
        )

    @mock.patch('awscli.customizations.codeartifact.login.is_macos', False)
    def test_swift_login_with_namespace_non_macos(self):
        cmdline = self._setup_cmd(tool='swift', include_namespace=True)
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(package_format='swift', result=result)
        self._assert_expiration_printed_to_stdout(result.stdout)
        self._assert_subprocess_execution(
            self._get_swift_commands(scope=self.namespace)
        )
        self._assert_netrc_has_expected_content()

    @mock.patch('awscli.customizations.codeartifact.login.is_macos', True)
    def test_swift_login_with_namespace_dry_run(self):
        cmdline = self._setup_cmd(
            tool='swift', include_namespace=True, dry_run=True
        )
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(package_format='swift', result=result)
        self._assert_dry_run_execution(
            self._get_swift_commands(scope=self.namespace), result.stdout
        )

    @mock.patch('awscli.customizations.codeartifact.login.is_macos', False)
    def test_swift_login_with_namespace_with_endpoint_type(self):
        cmdline = self._setup_cmd(
            tool='swift', include_namespace=True, include_endpoint_type=True
        )
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(
            package_format='swift', result=result, include_endpoint_type=True
        )
        self._assert_expiration_printed_to_stdout(result.stdout)
        self._assert_subprocess_execution(
            self._get_swift_commands(scope=self.namespace)
        )
        self._assert_netrc_has_expected_content()

    @mock.patch('awscli.customizations.codeartifact.login.is_macos', True)
    def test_swift_login_with_domain_owner_with_endpoint_type(self):
        cmdline = self._setup_cmd(
            tool='swift', include_domain_owner=True, include_endpoint_type=True
        )
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(
            package_format='swift',
            result=result,
            include_endpoint_type=True,
            include_domain_owner=True,
        )
        self._assert_expiration_printed_to_stdout(result.stdout)
        self._assert_subprocess_execution(
            self._get_swift_commands(token=self.auth_token)
        )

    def test_nuget_login_without_domain_owner_without_duration_seconds(self):
        cmdline = self._setup_cmd(tool='nuget')
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(package_format='nuget', result=result)
        self._assert_expiration_printed_to_stdout(result.stdout)
        self._assert_subprocess_execution(self._get_nuget_commands())

    def test_nuget_login_with_domain_owner_without_duration_seconds(self):
        cmdline = self._setup_cmd(tool='nuget', include_domain_owner=True)
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(
            package_format='nuget', include_domain_owner=True, result=result
        )
        self._assert_expiration_printed_to_stdout(result.stdout)
        self._assert_subprocess_execution(self._get_nuget_commands())

    def test_nuget_login_without_domain_owner_with_duration_seconds(self):
        cmdline = self._setup_cmd(tool='nuget', include_duration_seconds=True)
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(
            package_format='nuget',
            include_duration_seconds=True,
            result=result,
        )
        self._assert_expiration_printed_to_stdout(result.stdout)
        self._assert_subprocess_execution(self._get_nuget_commands())

    def test_nuget_login_with_domain_owner_duration_sections(self):
        cmdline = self._setup_cmd(
            tool='nuget',
            include_domain_owner=True,
            include_duration_seconds=True,
        )
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(
            package_format='nuget',
            include_domain_owner=True,
            include_duration_seconds=True,
            result=result,
        )
        self._assert_expiration_printed_to_stdout(result.stdout)
        self._assert_subprocess_execution(self._get_nuget_commands())

    def test_nuget_login_with_domain_owner_duration_endpoint_type(self):
        cmdline = self._setup_cmd(
            tool='nuget',
            include_domain_owner=True,
            include_duration_seconds=True,
            include_endpoint_type=True,
        )
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(
            package_format='nuget',
            include_domain_owner=True,
            include_duration_seconds=True,
            include_endpoint_type=True,
            result=result,
        )
        self._assert_expiration_printed_to_stdout(result.stdout)
        self._assert_subprocess_execution(self._get_nuget_commands())

    def test_nuget_login_without_domain_owner_dry_run(self):
        cmdline = self._setup_cmd(tool='nuget', dry_run=True)
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(package_format='nuget', result=result)
        self._assert_dry_run_execution(
            self._get_nuget_commands(), result.stdout
        )

    def test_nuget_login_with_domain_owner_dry_run(self):
        cmdline = self._setup_cmd(
            tool='nuget', include_domain_owner=True, dry_run=True
        )
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(
            package_format='nuget', include_domain_owner=True, result=result
        )
        self._assert_dry_run_execution(
            self._get_nuget_commands(), result.stdout
        )

    def test_nuget_login_with_duration_seconds_dry_run(self):
        cmdline = self._setup_cmd(
            tool='nuget', include_duration_seconds=True, dry_run=True
        )
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(
            package_format='nuget',
            include_duration_seconds=True,
            result=result,
        )
        self._assert_dry_run_execution(
            self._get_nuget_commands(), result.stdout
        )

    def test_nuget_login_with_domain_owner_duration_seconds_dry_run(self):
        cmdline = self._setup_cmd(
            tool='nuget',
            include_domain_owner=True,
            include_duration_seconds=True,
            dry_run=True,
        )
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(
            package_format='nuget',
            include_domain_owner=True,
            include_duration_seconds=True,
            result=result,
        )
        self._assert_dry_run_execution(
            self._get_nuget_commands(), result.stdout
        )

    def test_nuget_login_with_domain_owner_duration_seconds_with_endpoint_type_dryrun(
        self,
    ):
        cmdline = self._setup_cmd(
            tool='nuget',
            include_domain_owner=True,
            include_duration_seconds=True,
            dry_run=True,
            include_endpoint_type=True,
        )
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(
            package_format='nuget',
            include_domain_owner=True,
            include_duration_seconds=True,
            include_endpoint_type=True,
            result=result,
        )
        self._assert_dry_run_execution(
            self._get_nuget_commands(), result.stdout
        )

    @mock.patch('awscli.customizations.codeartifact.login.is_windows', True)
    def test_dotnet_login_without_domain_owner_without_duration_seconds(self):
        cmdline = self._setup_cmd(tool='dotnet')
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(package_format='nuget', result=result)
        self._assert_expiration_printed_to_stdout(result.stdout)
        self._assert_subprocess_execution(self._get_dotnet_commands())

    @mock.patch('awscli.customizations.codeartifact.login.is_windows', True)
    def test_dotnet_login_without_domain_owner_without_duration_seconds_with_endpoint_type(
        self,
    ):
        cmdline = self._setup_cmd(tool='dotnet', include_endpoint_type=True)
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(
            package_format='nuget', result=result, include_endpoint_type=True
        )
        self._assert_expiration_printed_to_stdout(result.stdout)
        self._assert_subprocess_execution(self._get_dotnet_commands())

    @mock.patch('awscli.customizations.codeartifact.login.is_windows', True)
    def test_dotnet_login_with_domain_owner_without_duration_seconds(self):
        cmdline = self._setup_cmd(tool='dotnet', include_domain_owner=True)
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(
            package_format='nuget', include_domain_owner=True, result=result
        )
        self._assert_expiration_printed_to_stdout(result.stdout)
        self._assert_subprocess_execution(self._get_dotnet_commands())

    @mock.patch('awscli.customizations.codeartifact.login.is_windows', True)
    def test_dotnet_login_without_domain_owner_with_duration_seconds(self):
        cmdline = self._setup_cmd(tool='dotnet', include_duration_seconds=True)
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(
            package_format='nuget',
            include_duration_seconds=True,
            result=result,
        )
        self._assert_expiration_printed_to_stdout(result.stdout)
        self._assert_subprocess_execution(self._get_dotnet_commands())

    @mock.patch('awscli.customizations.codeartifact.login.is_windows', True)
    def test_dotnet_login_with_domain_owner_duration_sections(self):
        cmdline = self._setup_cmd(
            tool='dotnet',
            include_domain_owner=True,
            include_duration_seconds=True,
        )
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(
            package_format='nuget',
            include_domain_owner=True,
            include_duration_seconds=True,
            result=result,
        )
        self._assert_expiration_printed_to_stdout(result.stdout)
        self._assert_subprocess_execution(self._get_dotnet_commands())

    @mock.patch('awscli.customizations.codeartifact.login.is_windows', True)
    def test_dotnet_login_without_domain_owner_dry_run(self):
        cmdline = self._setup_cmd(tool='dotnet', dry_run=True)
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(package_format='nuget', result=result)
        self._assert_dry_run_execution(
            self._get_dotnet_commands(), result.stdout
        )

    @mock.patch('awscli.customizations.codeartifact.login.is_windows', True)
    def test_dotnet_login_with_domain_owner_dry_run(self):
        cmdline = self._setup_cmd(
            tool='dotnet', include_domain_owner=True, dry_run=True
        )
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(
            package_format='nuget', include_domain_owner=True, result=result
        )
        self._assert_dry_run_execution(
            self._get_dotnet_commands(), result.stdout
        )

    @mock.patch('awscli.customizations.codeartifact.login.is_windows', True)
    def test_dotnet_login_with_duration_seconds_dry_run(self):
        cmdline = self._setup_cmd(
            tool='dotnet', include_duration_seconds=True, dry_run=True
        )
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(
            package_format='nuget',
            include_duration_seconds=True,
            result=result,
        )
        self._assert_dry_run_execution(
            self._get_dotnet_commands(), result.stdout
        )

    @mock.patch('awscli.customizations.codeartifact.login.is_windows', True)
    def test_dotnet_login_with_domain_owner_duration_seconds_dry_run(self):
        cmdline = self._setup_cmd(
            tool='dotnet',
            include_domain_owner=True,
            include_duration_seconds=True,
            dry_run=True,
        )
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(
            package_format='nuget',
            include_domain_owner=True,
            include_duration_seconds=True,
            result=result,
        )
        self._assert_dry_run_execution(
            self._get_dotnet_commands(), result.stdout
        )

    _NUGET_SOURCES_LIST_RESPONSE_WITH_EXTRA_NON_LIST_TEXT = b"""\
Welcome to dotnet 2.0!

Registered Sources:
  1. Source Name 1 [Enabled]
     https://source1.com/index.json
  2. ati-nugetserver [Disabled]
     http://atinugetserver-env.elasticbeanstalk.com/nuget
warn : You are running the 'list source' operation with an 'HTTP' source,
'ati-nugetserver' [http://atinugetserver-env..elasticbeanstalk.com/nuget]'.
Non-HTTPS access will be removed in a future version. Consider migrating
to an 'HTTPS' source."""

    @mock.patch('awscli.customizations.codeartifact.login.is_windows', True)
    def test_dotnet_login_sources_listed_with_extra_non_list_text(self):
        self.subprocess_check_output_patch.return_value = (
            self._NUGET_SOURCES_LIST_RESPONSE_WITH_EXTRA_NON_LIST_TEXT
        )

        cmdline = self._setup_cmd(tool='dotnet')
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(package_format='nuget', result=result)
        commands = [
            ['dotnet', 'nuget', 'list', 'source', '--format', 'detailed']
        ]
        self._assert_subprocess_check_output_execution(commands)

    @mock.patch('awscli.customizations.codeartifact.login.is_windows', True)
    def test_dotnet_login_sources_listed_with_extra_non_list_text_with_endpoint_type(
        self,
    ):
        self.subprocess_check_output_patch.return_value = (
            self._NUGET_SOURCES_LIST_RESPONSE_WITH_EXTRA_NON_LIST_TEXT
        )

        cmdline = self._setup_cmd(tool='dotnet', include_endpoint_type=True)
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(
            package_format='nuget', result=result, include_endpoint_type=True
        )
        commands = [
            ['dotnet', 'nuget', 'list', 'source', '--format', 'detailed']
        ]
        self._assert_subprocess_check_output_execution(commands)

    def test_npm_login_without_domain_owner(self):
        cmdline = self._setup_cmd(tool='npm')
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(package_format='npm', result=result)
        self._assert_expiration_printed_to_stdout(result.stdout)
        self._assert_subprocess_execution(self._get_npm_commands())

    def test_npm_login_without_domain_owner_dry_run(self):
        cmdline = self._setup_cmd(tool='npm', dry_run=True)
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(package_format='npm', result=result)
        self._assert_dry_run_execution(self._get_npm_commands(), result.stdout)

    def test_npm_login_always_auth_error_ignored(self):
        """Test login ignores error for always-auth.

        This test is for NPM version >= 9 where the support of 'always-auth'
        has been dropped. Running the command to set config gives a non-zero
        exit code. This is to make sure that login ignores that error and all
        other commands executes successfully.
        """

        def side_effect(command, capture_output, check):
            if any('always-auth' in arg for arg in command):
                raise subprocess.CalledProcessError(returncode=1, cmd=command)

            return mock.DEFAULT

        self.subprocess_mock.side_effect = side_effect
        cmdline = self._setup_cmd(tool='npm')
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_expiration_printed_to_stdout(result.stdout)
        self._assert_subprocess_execution(self._get_npm_commands())

    def test_npm_login_with_domain_owner(self):
        cmdline = self._setup_cmd(tool='npm', include_domain_owner=True)
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(
            package_format='npm',
            result=result,
            include_domain_owner=True,
            include_duration_seconds=False,
        )
        self._assert_expiration_printed_to_stdout(result.stdout)
        self._assert_subprocess_execution(self._get_npm_commands())

    def test_npm_login_with_domain_owner_endpoint_type(self):
        cmdline = self._setup_cmd(
            tool='npm', include_domain_owner=True, include_endpoint_type=True
        )
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(
            package_format='npm',
            result=result,
            include_domain_owner=True,
            include_duration_seconds=False,
            include_endpoint_type=True,
        )
        self._assert_expiration_printed_to_stdout(result.stdout)
        self._assert_subprocess_execution(self._get_npm_commands())

    def test_npm_login_with_domain_owner_duration(self):
        cmdline = self._setup_cmd(
            tool='npm',
            include_domain_owner=True,
            include_duration_seconds=True,
        )
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(
            package_format='npm',
            result=result,
            include_domain_owner=True,
            include_duration_seconds=True,
        )
        self._assert_expiration_printed_to_stdout(result.stdout)
        self._assert_subprocess_execution(self._get_npm_commands())

    def test_npm_login_with_domain_owner_dry_run(self):
        cmdline = self._setup_cmd(
            tool='npm', include_domain_owner=True, dry_run=True
        )
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(
            package_format='npm', result=result, include_domain_owner=True
        )
        self._assert_dry_run_execution(self._get_npm_commands(), result.stdout)

    def test_npm_login_with_namespace(self):
        cmdline = self._setup_cmd(tool='npm', include_namespace=True)
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(package_format='npm', result=result)
        self._assert_expiration_printed_to_stdout(result.stdout)
        self._assert_subprocess_execution(
            self._get_npm_commands(scope=f'@{self.namespace}')
        )

    def test_npm_login_with_namespace_dry_run(self):
        cmdline = self._setup_cmd(
            tool='npm', include_namespace=True, dry_run=True
        )
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(package_format='npm', result=result)
        self._assert_dry_run_execution(
            self._get_npm_commands(scope=f'@{self.namespace}'), result.stdout
        )

    def test_npm_login_with_namespace_endpoint_type_dry_run(self):
        cmdline = self._setup_cmd(
            tool='npm',
            include_namespace=True,
            dry_run=True,
            include_endpoint_type=True,
        )
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(
            package_format='npm', result=result, include_endpoint_type=True
        )
        self._assert_dry_run_execution(
            self._get_npm_commands(scope=f'@{self.namespace}'), result.stdout
        )

    def test_pip_login_without_domain_owner(self):
        cmdline = self._setup_cmd(tool='pip')
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(package_format='pypi', result=result)
        self._assert_expiration_printed_to_stdout(result.stdout)
        self._assert_subprocess_execution(self._get_pip_commands())

    def test_pip_login_without_domain_owner_dry_run(self):
        cmdline = self._setup_cmd(tool='pip', dry_run=True)
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(package_format='pypi', result=result)
        self._assert_dry_run_execution(self._get_pip_commands(), result.stdout)

    def test_pip_login_with_domain_owner(self):
        cmdline = self._setup_cmd(tool='pip', include_domain_owner=True)
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(
            package_format='pypi', result=result, include_domain_owner=True
        )
        self._assert_expiration_printed_to_stdout(result.stdout)
        self._assert_subprocess_execution(self._get_pip_commands())

    def test_pip_login_with_domain_owner_duration(self):
        cmdline = self._setup_cmd(
            tool='pip',
            include_domain_owner=True,
            include_duration_seconds=True,
        )
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(
            package_format='pypi',
            result=result,
            include_domain_owner=True,
            include_duration_seconds=True,
        )
        self._assert_expiration_printed_to_stdout(result.stdout)
        self._assert_subprocess_execution(self._get_pip_commands())

    def test_pip_login_with_domain_owner_duration_endpoint_type(self):
        cmdline = self._setup_cmd(
            tool='pip',
            include_domain_owner=True,
            include_duration_seconds=True,
            include_endpoint_type=True,
        )
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(
            package_format='pypi',
            result=result,
            include_domain_owner=True,
            include_duration_seconds=True,
            include_endpoint_type=True,
        )
        self._assert_expiration_printed_to_stdout(result.stdout)
        self._assert_subprocess_execution(self._get_pip_commands())

    def test_pip_login_with_domain_owner_dry_run(self):
        cmdline = self._setup_cmd(
            tool='pip', include_domain_owner=True, dry_run=True
        )
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(
            package_format='pypi', result=result, include_domain_owner=True
        )
        self._assert_dry_run_execution(self._get_pip_commands(), result.stdout)

    def test_pip_login_with_namespace(self):
        cmdline = self._setup_cmd(tool='pip', include_namespace=True)
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 255)
        self._assert_operations_called(package_format='pypi', result=result)
        self.assertIn(
            'Argument --namespace is not supported for pip', result.stderr
        )

    def test_pip_login_with_namespace_dry_run(self):
        cmdline = self._setup_cmd(
            tool='pip', include_namespace=True, dry_run=True
        )
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 255)
        self._assert_operations_called(package_format='pypi', result=result)
        self.assertIn(
            'Argument --namespace is not supported for pip', result.stderr
        )

    def test_pip_login_command_failed_auth_token_redacted(self):
        def side_effect(command, capture_output, check):
            raise subprocess.CalledProcessError(returncode=1, cmd=command)

        self.subprocess_mock.side_effect = side_effect
        cmdline = self._setup_cmd(tool='pip')
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 255)
        self.assertIn(
            "Command '['pip', 'config', 'set', 'global.index-url',"
            " 'https://aws:******@domain-domain-owner.codeartifact.aws.a2z.com/pypi/repository/simple/']'"
            " returned non-zero exit status 1.",
            result.stderr,
        )

    def test_twine_login_without_domain_owner(self):
        cmdline = self._setup_cmd(tool='twine')
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(package_format='pypi', result=result)
        self._assert_expiration_printed_to_stdout(result.stdout)
        with open(self.test_pypi_rc_path) as f:
            test_pypi_rc_str = f.read()

        self._assert_pypi_rc_has_expected_content(
            pypi_rc_str=test_pypi_rc_str,
            server='codeartifact',
            repo_url=self.endpoint,
            username='aws',
            password=self.auth_token,
        )

    def test_twine_login_without_domain_owner_dry_run(self):
        cmdline = self._setup_cmd(tool='twine', dry_run=True)
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(package_format='pypi', result=result)
        self.assertFalse(os.path.exists(self.test_pypi_rc_path))
        self._assert_pypi_rc_has_expected_content(
            pypi_rc_str=self._get_twine_commands(),
            server='codeartifact',
            repo_url=self.endpoint,
            username='aws',
            password=self.auth_token,
        )

    def test_twine_login_without_domain_owner_dry_run_endpoint_type(self):
        cmdline = self._setup_cmd(
            tool='twine', dry_run=True, include_endpoint_type=True
        )
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(
            package_format='pypi', result=result, include_endpoint_type=True
        )
        self.assertFalse(os.path.exists(self.test_pypi_rc_path))
        self._assert_pypi_rc_has_expected_content(
            pypi_rc_str=self._get_twine_commands(),
            server='codeartifact',
            repo_url=self.endpoint,
            username='aws',
            password=self.auth_token,
        )

    def test_twine_login_with_domain_owner(self):
        cmdline = self._setup_cmd(tool='twine', include_domain_owner=True)
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(
            package_format='pypi', result=result, include_domain_owner=True
        )
        self._assert_expiration_printed_to_stdout(result.stdout)

        with open(self.test_pypi_rc_path) as f:
            test_pypi_rc_str = f.read()

        self._assert_pypi_rc_has_expected_content(
            pypi_rc_str=test_pypi_rc_str,
            server='codeartifact',
            repo_url=self.endpoint,
            username='aws',
            password=self.auth_token,
        )

    def test_twine_login_with_domain_owner_duration(self):
        cmdline = self._setup_cmd(
            tool='twine',
            include_domain_owner=True,
            include_duration_seconds=True,
        )
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(
            package_format='pypi',
            result=result,
            include_domain_owner=True,
            include_duration_seconds=True,
        )
        self._assert_expiration_printed_to_stdout(result.stdout)

        with open(self.test_pypi_rc_path) as f:
            test_pypi_rc_str = f.read()

        self._assert_pypi_rc_has_expected_content(
            pypi_rc_str=test_pypi_rc_str,
            server='codeartifact',
            repo_url=self.endpoint,
            username='aws',
            password=self.auth_token,
        )

    def test_twine_login_with_domain_owner_dry_run(self):
        cmdline = self._setup_cmd(
            tool='twine', include_domain_owner=True, dry_run=True
        )
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 0)
        self._assert_operations_called(
            package_format='pypi', result=result, include_domain_owner=True
        )
        self.assertFalse(os.path.exists(self.test_pypi_rc_path))
        self._assert_pypi_rc_has_expected_content(
            pypi_rc_str=self._get_twine_commands(),
            server='codeartifact',
            repo_url=self.endpoint,
            username='aws',
            password=self.auth_token,
        )

    def test_twine_login_with_namespace(self):
        cmdline = self._setup_cmd(tool='twine', include_namespace=True)
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 255)
        self._assert_operations_called(package_format='pypi', result=result)
        self.assertIn(
            'Argument --namespace is not supported for twine', result.stderr
        )

    def test_twine_login_with_namespace_dry_run(self):
        cmdline = self._setup_cmd(
            tool='twine', include_namespace=True, dry_run=True
        )
        result = self.cli_runner.run(cmdline)
        self.assertEqual(result.rc, 255)
        self._assert_operations_called(package_format='pypi', result=result)
        self.assertFalse(os.path.exists(self.test_pypi_rc_path))
        self.assertIn(
            'Argument --namespace is not supported for twine', result.stderr
        )
